@@ -12,13 +12,16 @@ import tempfile
 
 from pkg_resources import resource_filename
 import formencode
-import kid
+# XXX: we need to make kid-support pluggable or such.
+try:
+    import kid
+except ImportError:
+    kid = None
 
 import turbogears
 import turbogears.i18n
 from turbogears import config
-from turbogears.toolbox.admi18n import pygettext, msgfmt, catalog
-from turbogears.toolbox.admi18n.catalog import normalize
+from turbogears.i18n.pygettext import pygettext, msgfmt, catalog
 from turbogears.command.base import silent_os_remove
 from turbogears.util import load_project_config, get_package_name
 
@@ -40,7 +43,7 @@ def copy_file(src, dest):
 _str_literal = r"""(?:'((?:[^']|\\')*)'|"((?:[^"]|\\")*)")"""
 _py_i18n_re = re.compile(r"\b_\s*\(\s*[uU]?[rR]?%s\s*\)" % _str_literal)
 _js_i18n_re = re.compile(r"\b_\s*\(\s*%s\s*\)" % _str_literal)
-
+_tmpl_expr_re = re.compile(r"\$\{.*?\}")
 
 class InternationalizationTool(object):
     """Manages i18n data via command-line interface.
@@ -80,6 +83,16 @@ Available commands:
         parser.add_option("-K", "--no-kid-support", default=True,
             action="store_false", dest="kid_support",
             help="Do not extract messages from Kid templates")
+        parser.add_option("-G", "--no-genshi-support", default=True,
+            action="store_false", dest="genshi_support",
+            help="Do not extract messages from Genshi templates")
+        parser.add_option("", "--loose-kid-support",
+            action="store_true", dest="loose_kid_support",
+            help="Extract ALL messages from Kid templates (this is default)")
+        parser.add_option("", "--strict-kid-support",
+            action="store_false", dest="loose_kid_support",
+            help="Extract only messages marked with lang attribute"
+                " from Kid templates")
         parser.add_option("", "--src-dir", default=None,
             action="store", dest="source_dir",
             help="Directory that contains source files")
@@ -94,7 +107,7 @@ Available commands:
         parser.add_option("-e", "--js-encoding",
             dest="js_encoding", default="utf-8", metavar="ENCODING",
             help="Encoding of JavaScript source files (default: UTF-8)")
-        parser.set_defaults(js_support=True)
+        parser.set_defaults(loose_kid_support=True, js_support=True)
         self.parser = parser
 
     def load_project_config(self):
@@ -273,6 +286,7 @@ for(var i in _messages) {
     def scan_source_files(self):
         source_files = []
         kid_files = []
+        genshi_files = []
         js_files = []
         srcdir = self.options.source_dir or get_package_name().split('.', 1)[0]
         print 'Scanning source directory', srcdir
@@ -286,6 +300,8 @@ for(var i in _messages) {
                     source_files.append(srcfile)
                 elif ext == '.kid':
                     kid_files.append(srcfile)
+                elif ext == '.html':
+                    genshi_files.append(srcfile)
                 elif ext == '.js':
                     js_files.append(srcfile)
                 else:
@@ -303,10 +319,22 @@ for(var i in _messages) {
         if not os.path.exists(tmp_potfile):
             raise ProgramError, 'pygettext failed'
         atexit.register(silent_os_remove, tmp_potfile)
+
         if kid_files and self.options.kid_support:
-            self.scan_kid_files(tmp_potfile, kid_files)
+            if not kid:
+                print "Kid not installed, no support for Kid templates."
+            else:
+                self.scan_kid_files(tmp_potfile, kid_files)
+
+        if genshi_files and self.options.genshi_support:
+            try:
+                self.scan_genshi_files(tmp_potfile, genshi_files)
+            except ImportError:
+                print "Genshi not installed, no support for Genshi templates."
+
         if js_files and self.options.js_support:
             self.scan_js_files(tmp_potfile, js_files)
+
         potfile = self.get_potfile_path()
         if os.path.isfile(potfile):
             bakfile = potfile.replace('.pot', '.bak')
@@ -316,7 +344,54 @@ for(var i in _messages) {
         os.rename(tmp_potfile, potfile)
         print 'Message templates written to', potfile
 
+    def scan_genshi_files(self, potfile, files):
+        """Extract strings from Genhsi templates and write to pot file.
+
+        Configuration settings:
+
+        `genshi.i18n.ignore_tags` -- `['script', 'style']`
+            List of element names. Content inside elements named in this list
+            is not extracted as translatable text. Can be a space-separated
+            string or a list of string.
+        `genshi.i18n.include_attrs` -- `['abbr', 'alt', 'label', 'prompt',
+                'standby', 'summary', 'title']`
+            List of attribute names. Only values of the attributes named in
+            this list are extracted as translatable text. Can be a
+            space-separated string or a list os strings.
+
+        See http://genshi.edgewall.org/wiki/Documentation/0.5.x/i18n.html for
+        more information.
+
+        """
+
+        extract_strings = turbogears.i18n.pygettext.pygettext.extract_genshi_strings
+        messages = []
+        options = {}
+        for opt in ('include_attrs', 'ignore_tags'):
+            val = config.get('genshi.i18n.' + opt)
+            if val:
+                options[opt] = val
+
+        for fname in files:
+            print 'Working on', fname
+            for msg in extract_strings(fname, options or None):
+                # Genshi 0.6 adds additional 4th element, so we take
+                # care to stay compatible
+                lineno, text = msg[0], msg[2]
+                # Split into substrings around inline Python expressions.
+                # XXX: strings with inline Python expressions only get
+                # translated properly when the expression is at the start or
+                # end of the string.
+                for substring in _tmpl_expr_re.split(text):
+                    messages.append((lineno, fname, substring.strip()))
+
+        self._write_potfile_entries(potfile, messages)
+
     def scan_kid_files(self, potfile, files):
+        """Extract strings from Genhsi templates and write to pot file."""
+
+        # XXX: This duplicates functionality already present in
+        #   turbogears.i18n.pygettext! This should be recfactored.
         messages = []
         tags_to_ignore = ['script', 'style']
         keys = []
@@ -377,14 +452,7 @@ for(var i in _messages) {
                     if sentinel is None and el.strip():
                         process_text(False, el, tag)
             fh.close()
-        fd = open(potfile, 'at+')
-        for tag, fname, text in messages:
-            text = normalize(text.encode('utf-8'))
-            print >> fd, '#: %s:%s' % (fname, tag)
-            print >> fd, 'msgid %s' % text
-            print >> fd, 'msgstr ""'
-            print >> fd, ''
-        fd.close()
+        self._write_potfile_entries(potfile, messages)
 
     def get_strings_in_js(self, fname):
         messages = []
