@@ -1,25 +1,27 @@
 """Classes and methods for TurboGears controllers."""
 
 import logging
-import re
 import urllib
 import urlparse
+import time
 import types
+
 from itertools import izip
-from dispatch import generic, strategy, functions
 from inspect import isclass
+
 import cherrypy
 from cherrypy import request, response
+
+from peak.rules import abstract, always_overrides, Method, NoApplicableMethods
+
 import turbogears.util as tg_util
 from turbogears import view, database, errorhandling, config
 from turbogears.decorator import weak_signature_decorator
-from turbogears.validators import Invalid
 from turbogears.errorhandling import error_handler, exception_handler
-
+from turbogears.validators import Invalid
 
 log = logging.getLogger("turbogears.controllers")
 
-
 if config.get("session_filter.on", None):
     if config.get("session_filter.storage_type", None) == "PostgreSQL":
         import psycopg2
@@ -27,17 +29,18 @@ if config.get("session_filter.on", None):
                 {'session_filter.get_db': psycopg2.connect(
                     psycopg2.get('sessions.postgres.dsn'))
                     })
-    # support for mysql/sqlite/etc here
+    # XXX: support for mysql/sqlite/etc here
 
 
-def _process_output(output, template, format, content_type,
-        mapping, fragment=False):
+def _process_output(output, template, format, content_type, fragment=False,
+                    **options):
     """Produce final output form from data returned from a controller method.
 
     See the expose() arguments for more info since they are the same.
 
     """
     if isinstance(output, dict):
+        # import this here to prevent circular import in widgets.forms
         from turbogears.widgets import js_location
 
         css = tg_util.setlike()
@@ -48,35 +51,39 @@ def _process_output(output, template, format, content_type,
         if config.get("tg.mochikit_all", False):
             include_widgets_lst.insert(0, 'turbogears.mochikit')
 
-        for i in include_widgets_lst:
-            widget = tg_util.load_class(i)
+        for name in include_widgets_lst:
+            widget = tg_util.load_class(name)
             if isclass(widget):
                 widget = widget()
-            include_widgets["tg_%s" % i.split(".")[-1]] = widget
-            for script in widget.retrieve_javascript():
-                if hasattr(script, "location"):
-                    js[script.location].add(script)
-                else:
-                    js[js_location.head].add(script)
-            css.add_all(widget.retrieve_css())
-
-        for value in output.itervalues():
-            if hasattr(value, "retrieve_css"):
-                retrieve = getattr(value, "retrieve_css")
-                if callable(retrieve):
-                    css.add_all(value.retrieve_css())
-            if hasattr(value, "retrieve_javascript"):
-                retrieve = getattr(value, "retrieve_javascript")
-                if callable(retrieve):
-                    for script in value.retrieve_javascript():
-                        if hasattr(script, "location"):
-                            js[script.location].add(script)
-                        else:
-                            js[js_location.head].add(script)
+            # XXX: widgets with same base name will override each other
+            include_widgets["tg_%s" % name.split(".")[-1]] = widget
         output.update(include_widgets)
+
+        # collect JS/CSS resources from widgets in the output dict or
+        # tg.include_widgets
+        for value in output.itervalues():
+            if hasattr(value, "retrieve_resources"):
+                # it's a tw widget --> ToscaWidgets takes care of JS/CSS
+                # resource inclusion
+                continue
+            else:
+                try:
+                    css_resources = value.retrieve_css()
+                except (AttributeError, TypeError):
+                    css_resources = []
+                try:
+                    js_resources = value.retrieve_javascript()
+                except (AttributeError, TypeError):
+                    js_resources = []
+            css.add_all(css_resources)
+            for script in js_resources:
+                location = getattr(script, "location", js_location.head)
+                js[location].add(script)
+        css.sort(key=lambda obj:  getattr(obj, "order", 0))
         output["tg_css"] = css
         for location in iter(js_location):
-            output["tg_js_%s" % str(location)] = js[location]
+            js[location].sort(key=lambda obj:  getattr(obj, "order", 0))
+            output["tg_js_%s" % location] = js[location]
 
         tg_flash = _get_flash()
         if tg_flash:
@@ -84,8 +91,7 @@ def _process_output(output, template, format, content_type,
 
         headers = {'Content-Type': content_type}
         output = view.render(output, template=template, format=format,
-                    mapping=mapping, headers=headers,
-                    fragment=fragment)
+                             headers=headers, fragment=fragment, **options)
         content_type = headers['Content-Type']
 
     if content_type:
@@ -142,7 +148,7 @@ def validate(form=None, validators=None,
     """
     def entangle(func):
         if callable(form) and not hasattr(form, "validate"):
-            init_form = lambda self: form(self)
+            init_form = form
         else:
             init_form = lambda self: form
 
@@ -198,20 +204,13 @@ def validate(form=None, validators=None,
     return weak_signature_decorator(entangle)
 
 
-class CustomDispatch(functions.GenericFunction):
+class First(Method):
+    """Resolve ambiguousness by calling the first method."""
+    def merge(self, other):
+        return self
 
-    def combine(self, cases):
-        strict = [strategy.ordered_signatures, strategy.safe_methods]
-        cases = strategy.separate_qualifiers(
-            cases,
-            primary = strict,
-        )
-        primary = strategy.method_chain(cases.get('primary', []))
-        if type(primary) != types.FunctionType:
-            for i in primary:
-                for y in i:
-                    return y[1]
-        return primary
+always_overrides(First, Method)
+first = First.make_decorator('first')
 
 
 def _add_rule(_expose, found_default, as_format, accept_format, template,
@@ -231,13 +230,12 @@ def _add_rule(_expose, found_default, as_format, accept_format, template,
             ', "default") == "default")' % accept_format)
     rule = " or ".join(ruleparts)
     log.debug("Generated rule %s", rule)
-    _expose.when(rule)(rulefunc)
-
+    first(_expose, rule)(rulefunc)
     return found_default
 
 
 def _build_rules(func):
-    [generic(CustomDispatch)]
+    @abstract()
     def _expose(func, accept, allow_json, *args, **kw):
         pass
 
@@ -245,11 +243,11 @@ def _build_rules(func):
         log.debug("Adding allow_json rule: "
             'allow_json and (kw.get("tg_format", None) == "json"'
             ' or accept in ("application/json", "text/javascript"))')
-        _expose.when('allow_json and (kw.get("tg_format", None) == "json"'
+        first(_expose, 'allow_json and (kw.get("tg_format", None) == "json"'
             ' or accept in ("application/json", "text/javascript"))')(
             lambda _func, accept, allow_json, *args, **kw:
                 _execute_func(_func, "json", "json", "application/json",
-                    None, False, args, kw))
+                    False, {}, args, kw))
 
     found_default = False
     for ruleinfo in func._ruleinfo:
@@ -258,9 +256,8 @@ def _build_rules(func):
     func._expose = _expose
 
 
-def expose(template=None, validators=None, allow_json=None, html=None,
-           format=None, content_type=None, inputform=None, fragment=False,
-           as_format="default", mapping=None, accept_format=None):
+def expose(template=None, allow_json=None, format=None, content_type=None,
+           fragment=False, as_format="default", accept_format=None, **options):
     """Exposes a method to the web.
 
     By putting the expose decorator on a method, you tell TurboGears that
@@ -300,40 +297,37 @@ def expose(template=None, validators=None, allow_json=None, html=None,
     <a href="tests/test_expose-source.html">test_expose.py</a> suite
     for more examples.
 
-    @param template "templateengine:dotted.reference" reference along the
+    @param template: "templateengine:dotted.reference" reference along the
             Python path for the template and the template engine. For
             example, "kid:foo.bar" will have Kid render the bar template in
             the foo package.
-    @keyparam format format for the template engine to output (if the
+    @keyparam format: format for the template engine to output (if the
             template engine can render different formats. Kid, for example,
             can render "html", "xml" or "xhtml")
-    @keyparam content_type sets the content-type http header
-    @keyparam allow_json allow the function to be exposed as json
-    @keyparam fragment for template engines (like Kid) that generate
+    @keyparam content_type: sets the content-type http header
+    @keyparam allow_json: allow the function to be exposed as json
+    @keyparam fragment: for template engines (like Kid) that generate
             DOCTYPE declarations and the like, this is a signal to
             just generate the immediate template fragment. Use this
             if you're building up a page from multiple templates or
             going to put something onto a page with .innerHTML.
-    @keyparam mapping mapping with options that are sent to the template
-            engine
-    @keyparam as_format designates which value of tg_format will choose
+    @keyparam as_format: designates which value of tg_format will choose
             this expose.
-    @keyparam accept_format which value of an Accept: header will
+    @keyparam accept_format: which value of an Accept: header will
             choose this expose.
-    @keyparam html deprecated in favor of template
-    @keyparam validators deprecated. Maps argument names to validator
-            applied to that arg
-    @keyparam inputform deprecated. A form object that generates the
-            input to this method
+
+    All additional keyword arguments are passed as keyword args to the render
+    method of the template engine.
 
     """
-    if html:
-        template = html
     if not template:
         template = format
-    if format == "json" or (format is None and template is None):
+
+    if format == "json" or (format is None and template is None
+            and (allow_json is None or allow_json)):
         template = "json"
         allow_json = True
+
     if content_type is None:
         content_type = config.get("tg.content_type", None)
 
@@ -346,18 +340,36 @@ def expose(template=None, validators=None, allow_json=None, html=None,
             "content-type: %s", template, format, allow_json, content_type)
         if not getattr(func, "exposed", False):
             def expose(func, *args, **kw):
+                request.tg_template_enginename = view.base._choose_engine(template)[2]
                 accept = request.headers.get('Accept', "").lower()
                 accept = tg_util.simplify_http_accept_header(accept)
                 if not hasattr(func, "_expose"):
                     _build_rules(func)
-                if hasattr(request, "in_transaction"):
-                    output = func._expose(func, accept, func._allow_json,
+                try:
+                    if hasattr(request, "in_transaction"):
+                        output = func._expose(func, accept, func._allow_json,
+                                    *args, **kw)
+                    else:
+                        request.in_transaction = True
+                        output = database.run_with_transaction(
+                                func._expose, func, accept, func._allow_json,
                                 *args, **kw)
-                else:
-                    request.in_transaction = True
-                    output = database.run_with_transaction(
-                            func._expose, func, accept, func._allow_json,
-                            *args, **kw)
+                except NoApplicableMethods, e:
+                    args = e.args # args from the last generic function call
+                    if (args and args[0] and isinstance(args[0], tuple)
+                            and args[0][0] is func):
+                        # The error refers to our call above. This means that
+                        # no suitable controller method was found (probably due
+                        # to wrong parameters). So we will raise a "not found"
+                        # error unless a specific error status was already set
+                        # (e.g. "unauthorized" was set by the identity provider):
+                        status = cherrypy.response.status
+                        if status and status // 100 == 4:
+                            raise cherrypy.HTTPError(status)
+                        raise cherrypy.NotFound
+                    # If the error was raised elsewhere inside the controller,
+                    # handle it like all other exceptions ("server error"):
+                    raise
                 return output
             func.exposed = True
             func._ruleinfo = []
@@ -368,56 +380,70 @@ def expose(template=None, validators=None, allow_json=None, html=None,
 
         func._ruleinfo.insert(0, dict(as_format=as_format,
             accept_format=accept_format, template=template,
-            rulefunc = lambda _func, accept, allow_json, *args, **kw:
+            rulefunc=lambda _func, accept, allow_json, *args, **kw:
                 _execute_func(_func, template, format, content_type,
-                    mapping, fragment, args, kw)))
+                              fragment, options, args, kw)))
 
         if allow_json:
             func._allow_json = True
 
-        if inputform or validators:
-            import warnings
-            warnings.warn(
-                "Use a separate decorator validate() rather than passing "
-                "arguments validators and/or inputform to decorator "
-                "expose().",
-                DeprecationWarning, 2)
-            func = validate(form=inputform, validators=validators)(func)
-
         return expose
     return weak_signature_decorator(entangle)
 
 
-def _execute_func(func, template, format, content_type, mapping, fragment, args, kw):
+def _execute_func(func, template, format, content_type, fragment, options,
+                  args, kw):
     """Call controller method and process it's output."""
+
     if config.get("tg.strict_parameters", False):
         tg_util.remove_keys(kw, ["tg_random", "tg_format"]
             + config.get("tg.ignore_parameters", []))
+
     else:
         # get special parameters used by upstream decorators like paginate
         try:
             tg_kw = dict([(k, v) for k, v in kw.items() if k in func._tg_args])
+
         except AttributeError:
             tg_kw = {}
+
         # remove excessive parameters
         args, kw = tg_util.adapt_call(func, args, kw)
         # add special parameters again
         kw.update(tg_kw)
+
     if config.get('server.environment', 'development') == 'development':
         # Only output this in development mode: If it's a field storage object,
         # this means big memory usage, and we don't want that in production
         log.debug("Calling %s with *(%s), **(%s)", func, args, kw)
+
     output = errorhandling.try_call(func, *args, **kw)
-    assert isinstance(output, (basestring, dict, list, types.GeneratorType)), \
-           "Method %s.%s() returned unexpected output. Output should " \
-           "be of type basestring, dict, list or generator." % (
-            args[0].__class__.__name__, func.__name__)
-    if isinstance(output, dict):
-        template = output.pop("tg_template", template)
-        format = output.pop("tg_format", format)
-    if template and template.startswith("."):
-        template = func.__module__[:func.__module__.rfind('.')]+template
-    return _process_output(output, template, format, content_type, mapping, fragment)
+
+    if str(getattr(response, 'status', '')).startswith('204'):
+        # HTTP status 204 indicates a response with no body
+        # so there should be no content type header
+        try:
+            del response.headers['Content-Type']
+        except (AttributeError, KeyError):
+            pass
+        return
+
+    else:
+        assert isinstance(output,
+            (basestring, dict, list, types.GeneratorType)), (
+            "Method %s.%s() returned unexpected output. Output should "
+            "be of type basestring, dict, list or generator." % (
+             args[0].__class__.__name__, func.__name__))
+
+        if isinstance(output, dict):
+            template = output.pop("tg_template", template)
+            format = output.pop("tg_format", format)
+
+        if template and template.startswith("."):
+            template = func.__module__[:func.__module__.rfind('.')]+template
+
+        return _process_output(output, template, format, content_type,
+                               fragment, **options)
 
 
 def flash(message):
@@ -457,8 +483,9 @@ def _get_flash():
 class Controller(object):
     """Base class for a web application's controller.
 
-    Currently, this provides positional parameters functionality
-    via a standard default method.
+    It is important that your controllers inherit from this class, otherwise
+    ``identity.SecureResource`` and ``identity.SecureObject`` will not work
+    correctly.
 
     """
 
@@ -466,8 +493,8 @@ class Controller(object):
 class RootController(Controller):
     """Base class for the root of a web application.
 
-    Your web application should have one of these. The root of
-    your application is used to compute URLs used by your app.
+    Your web application must have one of these. The root of your application
+    is used to compute URLs used by your app.
 
     """
     is_app_root = True
@@ -481,7 +508,10 @@ class RootController(Controller):
 
     accesslog = logging.getLogger("turbogears.access")
     def _cp_log_access(self):
-        tmpl = '%(h)s %(l)s %(u)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
+        # Conforms to Combined Log Format
+        # http://httpd.apache.org/docs/2.2/logs.html#combined
+        tmpl = '%(host)s %(ident)s %(authuser)s [%(date)s] "%(request)s" ' \
+            '%(status)s %(bytes)s "%(referrer)s" "%(useragent)s"'
         try:
             username = request.user_name
             if username:
@@ -492,36 +522,92 @@ class RootController(Controller):
                 username = "-"
         except AttributeError:
             username = "-"
-        s = tmpl % {'h': request.remote_host,
-                   'l': '-',
-                   'u': username,
-                   'r': request.requestLine,
-                   's': response.status.split(" ", 1)[0],
-                   'b': response.headers.get('Content-Length',
-                            '') or "-",
-                   'f': request.headers.get('referer', ''),
-                   'a': request.headers.get('user-agent', ''),
+        request_date = time.strftime('%d/%b/%Y:%H:%M:%S +0000',time.gmtime())
+        request_info = {
+            'host': request.headers.get('X-Forwarded-For')
+                or request.remote_host or request.remote_addr,
+            'ident': '-',
+            'authuser': username,
+            'date': request_date,
+            'request': request.requestLine,
+            'status': response.status.split(" ", 1)[0],
+            'bytes': response.headers.get('Content-Length') or "-",
+            'referrer': request.headers.get('referer', ""),
+            'useragent': request.headers.get('user-agent', ""),
         }
-        self.accesslog.info(s)
+        self.accesslog.info(tmpl, request_info)
 
 Root = RootController
 
 
-def url(tgpath, tgparams=None, **kw):
-    """Computes URLs.
+class ExposedDescriptor(object):
+    """Descriptor used by RESTMethod to tell if it is exposed."""
 
-    tgpath can be a list or a string. If the path is absolute (starts
-    with a "/"), the server.webpath, SCRIPT_NAME and the approot of the
-    application are prepended to the path. In order for the approot to
-    be detected properly, the root object should extend
-    controllers.RootController.
+    def __get__(self, obj, cls=None):
+        """Return True if object has a method for HTTP method of current request
+        """
+        if cls is None: cls = obj
+        cp_methodname = cherrypy.request.method
+        methodname = cp_methodname.lower()
+        method = getattr(cls, methodname, None)
+        if callable(method) and getattr(method, 'exposed', False):
+            return True
+        raise cherrypy.HTTPError(405, '%s not allowed on %s' % (
+            cp_methodname, cherrypy.request.browser_url))
+
+
+class RESTMethod(object):
+    """Allow REST style dispatch based on different HTTP methods.
+
+    For an elaborate usage example see turbogears.tests.test_restmethod.
+
+    In short, instead of an exposed method, you define a sub-class of
+    RESTMethod inside the controller class and inside this class you define
+    exposed methods named after each HTTP method that should be supported.
+
+    Example::
+
+        class Controller(controllers.Controller):
+
+            class article(copntrollers.RESTMethod):
+                @expose()
+                def get(self, id):
+                    ...
+
+                @expose()
+                def post(self, id):
+                    ...
+
+    """
+    exposed = ExposedDescriptor()
+
+    def __init__(self, *l, **kw):
+        methodname = cherrypy.request.method.lower()
+        self.result = getattr(self, methodname)(*l, **kw)
+
+    def __iter__(self):
+        return iter(self.result)
+
+
+def url(tgpath, tgparams=None, **kw):
+    """Computes relocatable URLs.
+
+    tgpath can be a list or a string. If the path is absolute (starts with a
+    "/"), the server.webpath, SCRIPT_NAME and the approot of the application
+    are prepended to the path. In order for the approot to be detected
+    properly, the root object must extend controllers.RootController.
 
     Query parameters for the URL can be passed in as a dictionary in
-    the second argument *or* as keyword parameters.
+    the second argument and/or as keyword parameters where keyword args
+    overwrite entries in the dictionary.
 
     Values which are a list or a tuple are used to create multiple
     key-value pairs.
 
+    tgpath may also already contain a (properly escaped) query string seperated
+    by a question mark ('?'), in which case additional query params are
+    appended.
+
     """
     if not isinstance(tgpath, basestring):
         tgpath = '/'.join(list(tgpath))
@@ -565,6 +651,59 @@ def url(tgpath, tgparams=None, **kw):
             tgpath += '?' + query_string
     return tgpath
 
+def get_server_name():
+    """Return name of the server this application runs on.
+
+    Respects 'Host' and 'X-Forwarded-Host' header.
+
+    See the docstring of the 'absolute_url' function for more information.
+
+    """
+    get = config.get
+    h = request.headers
+    host = get('tg.url_domain') or h.get('X-Forwarded-Host', h.get('Host'))
+    if not host:
+        host = '%s:%s' % (get('server.socket_host', 'localhost'),
+            get('server.socket_port', 8080))
+    return host
+
+def absolute_url(tgpath='/', params=None, **kw):
+    """Return absolute URL (including schema and host to this server).
+
+    Tries to account for 'Host' header and reverse proxying
+    ('X-Forwarded-Host').
+
+    The host name is determined this way:
+
+    * If the config setting 'tg.url_domain' is set and non-null, use this value.
+    * Else, if the 'base_url_filter.use_x_forwarded_host' config setting is
+      True, use the value from the 'Host' or 'X-Forwarded-Host' request header.
+    * Else, if config setting 'base_url_filter.on' is True and
+      'base_url_filter.base_url' is non-null, use its value for the host AND
+      scheme part of the URL.
+    * As a last fallback, use the value of 'server.socket_host' and
+      'server.socket_port' config settings (defaults to 'localhost:8080').
+
+    The URL scheme ('http' or 'http') used is determined in the following way:
+
+    * If 'base_url_filter.base_url' is used, use the scheme from this URL.
+    * If there is a 'X-Use-SSL' request header, use 'https'.
+    * Else, if the config setting 'tg.url_scheme' is set, use its value.
+    * Else, use the value of 'cherrypy.request.scheme'.
+
+    """
+    get = config.get
+    use_xfh = get('base_url_filter.use_x_forwarded_host', False)
+    if request.headers.get('X-Use-SSL'):
+        scheme = 'https'
+    else:
+        scheme = get('tg.url_scheme')
+    if not scheme:
+        scheme = request.scheme
+    base_url = '%s://%s' % (scheme, get_server_name())
+    if get('base_url_filter.on', False) and not use_xfh:
+        base_url = get('base_url_filter.base_url').rstrip('/')
+    return '%s%s' % (base_url, url(tgpath, params, **kw))
 
 def check_app_root():
     """Sets request.app_root if needed."""
@@ -617,9 +756,11 @@ def redirect(redirect_path, redirect_params=None, **kw):
 
 __all__ = [
     "Controller",
+    "absolute_url",
     "error_handler",
     "exception_handler",
     "expose",
+    "get_server_name",
     "flash",
     "redirect",
     "Root",
