@@ -1,87 +1,195 @@
 # coding=UTF-8
-
 import re
 import unittest
 import urllib
 import base64
-
 import formencode
 import cherrypy
-
 from turbogears import testutil, database, identity, config, startup, expose
 from turbogears.controllers import Controller, RootController
-from turbogears.identity import SecureResource, \
-    has_permission, in_group, not_anonymous
+from turbogears.identity import (SecureResource, Predicate,
+    in_group, in_all_groups, in_any_group, not_anonymous,
+    has_permission, has_all_permissions, has_any_permission,
+    from_host, from_any_host, Any, All, NotAny)
+from turbogears.identity.conditions import _remoteHost
 from turbogears.identity.soprovider import TG_User, TG_Group, TG_Permission
+from turbogears.identity.exceptions import (IdentityConfigurationException,
+    RequestRequiredException)
+from turbogears.identity.visitor import IdentityVisitPlugin
 
-# hub = database.AutoConnectHub("sqlite:///:memory:")
+#hub = database.AutoConnectHub("sqlite:///:memory:")
 hub = database.PackageHub("turbogears.identity")
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
-
 
 def mycustomencrypt(password):
+    """A custom password encryption function."""
     return password + '_custom'
 
 
+class MockIdentity(object):
+    """Identity mock object."""
+
+    anonymous = False
+    groups = ('admin', 'edit')
+    permissions = ('read', 'write')
+
+mock_identity = MockIdentity()
+
+
+class TestPredicates(unittest.TestCase):
+
+    def met(self, predicate):
+        return predicate.eval_with_object(mock_identity)
+
+    def test_in_group(self):
+        """Test the predicate for requiring a group."""
+        assert self.met(in_group('admin'))
+        assert not self.met(in_group('guest'))
+
+    def test_in_all_groups(self):
+        """Test the predicate for requiring a number of group."""
+        assert self.met(in_all_groups('admin', 'edit'))
+        assert not self.met(in_all_groups('admin', 'guest', 'edit'))
+
+    def test_in_any_group(self):
+        """Test the predicate for requiring at least one group."""
+        assert self.met(in_any_group('guest', 'edit', 'user'))
+        assert not self.met(in_all_groups('guest', 'user'))
+
+    def test_anonymous(self):
+        """Test predicate for checking anonymous visitors."""
+        assert self.met(not_anonymous())
+
+    def test_has_permission(self):
+        """Test predicate for checking particular permissions."""
+        assert self.met(has_permission('read'))
+        assert not self.met(has_permission('delete'))
+
+    def test_has_all_permissions(self):
+        """Test the predicate for requiring a number of permissions."""
+        assert self.met(has_all_permissions('read', 'write'))
+        assert not self.met(has_all_permissions('read', 'delete', 'write'))
+
+    def test_has_any_permission(self):
+        """Test the predicate for requiring at least one permission."""
+        assert self.met(has_any_permission('delete', 'write', 'update'))
+        assert not self.met(has_any_permission('delete', 'update'))
+
+    def test_any(self):
+        """Test the Any predicate."""
+        yes = in_group('edit')
+        no1, no2 = in_group('guest'), in_group('user')
+        met = self.met
+        assert met(Any(yes))
+        assert not met(Any(no1, no2))
+        assert met(Any(no1, yes, no2))
+
+    def test_all(self):
+        """Test the All predicate."""
+        yes1, yes2 = in_group('admin'), in_group('edit')
+        no = in_group('guest')
+        met = self.met
+        assert not met(All(no))
+        assert met(All(yes1, yes2))
+        assert not met(All(yes1, no, yes2))
+
+    def test_not_any(self):
+        """Test the Not Any predicate."""
+        yes = in_group('edit')
+        no1, no2 = in_group('guest'), in_group('user')
+        met = self.met
+        assert not met(NotAny(yes))
+        assert met(NotAny(no1, no2))
+        assert not met(NotAny(no1, yes, no2))
+
+    def test_remote_host_request_required(self):
+        """Test that _remoteHost raises exception when request is not available.
+        """
+        self.assertRaises(RequestRequiredException, _remoteHost)
+
+
 class RestrictedArea(Controller, SecureResource):
 
     require = in_group('peon')
 
-    [expose()]
+    @expose()
     def index(self):
         return "restricted_index"
 
-    [expose()]
-    [identity.require(in_group('admin'))]
+    @expose()
+    @identity.require(in_group('admin'))
     def in_admin_group(self):
         return 'in_admin_group'
 
-    [expose()]
-    [identity.require(in_group('other'))]
+    @expose()
+    @identity.require(in_group('other'))
     def in_other_group(self):
         return 'in_other_group'
 
-    [expose()]
+    @expose()
     def in_admin_group_explicit_check(self):
         if 'admin' not in identity.current.groups:
             raise identity.IdentityFailure("You need to be an Admin")
         else:
             return 'in_admin_group'
 
-    [expose()]
+    @expose()
     def in_other_group_explicit_check(self):
         if 'other' not in identity.current.groups:
             raise identity.IdentityException
         else:
             return 'in_other_group'
 
+    @expose(format='json')
+    def json(self):
+        return dict(json="restricted_json")
+
 
 class IdentityRoot(RootController):
 
-    [expose()]
+    @expose()
     def index(self):
-        pass
+        return dict()
 
-    [expose()]
+    @expose(format='html')
     def identity_failed(self, **kw):
-        cherrypy.response.status = 401
+        """Identity failure - this usually returns a login form."""
         return 'identity_failed_answer'
 
-    [expose()]
-    [identity.require(not_anonymous())]
+    @expose()
+    @identity.require(not_anonymous())
     def logged_in_only(self):
         return 'logged_in_only'
 
-    [expose()]
-    [identity.require(in_group('peon'))]
+    @expose()
+    @identity.require(in_group('peon'))
     def in_peon_group(self):
+        user = TG_User.by_user_name("samIam")
+        group_ids = set((TG_Group.by_group_name("peon").id,
+            TG_Group.by_group_name("other").id))
+        assert identity.current.user == user
+        assert identity.current.user_name == user.user_name
+        assert identity.current.user_id == user.id
+        assert identity.current.groups == set(('peon', 'other'))
+        assert identity.current.group_ids == group_ids
+        assert "samIam" == cherrypy.serving.request.identity.user_name
         return 'in_peon_group'
 
-    [expose()]
+    @expose()
+    def remote_ip(self):
+        return dict(remote_ip=_remoteHost())
+
+    @expose()
+    @identity.require(from_host('127.0.0.1'))
+    def from_localhost(self):
+        return "localhost_only"
+
+    @expose()
+    @identity.require(from_any_host(('127.0.0.1', '127.0.0.2')))
+    def from_any_host(self):
+        return "hosts_on_list_only"
+
+    @expose()
     def test_exposed_require(self):
         if not hasattr(self.in_peon_group, '_require'):
             return 'no _require attr'
@@ -91,41 +199,41 @@ class IdentityRoot(RootController):
             return 'not correct group name'
         return '_require is exposed'
 
-    [expose()]
-    [identity.require(in_group('admin'))]
+    @expose()
+    @identity.require(in_group('admin'))
     def in_admin_group(self):
         return 'in_admin_group'
 
-    [expose()]
-    [identity.require(has_permission('chops_wood'))]
+    @expose()
+    @identity.require(has_permission('chops_wood'))
     def has_chopper_permission(self):
         return 'has_chopper_permission'
 
-    [expose()]
-    [identity.require(has_permission('bosses_people'))]
+    @expose()
+    @identity.require(has_permission('bosses_people'))
     def has_boss_permission(self):
         return "has_boss_permission"
 
-    [expose()]
+    @expose()
     def logout(self):
         identity.current.logout()
         return "logged out"
 
-    [expose()]
-    [identity.require(not_anonymous())]
+    @expose()
+    @identity.require(not_anonymous())
     def user_email(self):
         return identity.current.user.email_address
 
     peon_area = RestrictedArea()
 
-    [expose()]
+    @expose()
     def new_user_setup(self, user_name, password):
         return '%s %s' % (user_name, password)
 
     _test_encoded_params = ('b=krümel&d.a=klöße1')
 
-    [expose()]
-    [identity.require(not_anonymous())]
+    @expose()
+    @identity.require(not_anonymous())
     def test_params(self, **kwargs):
         params = self._test_encoded_params
         # formencode's variable_decode create a datastructure
@@ -146,28 +254,44 @@ class IdentityRoot(RootController):
             return 'wrong params: %s\nexpected unicode objects' \
                 ' for all strings' % cherrypy.request.params
 
+    @expose()
+    def is_anonymous(self):
+        assert cherrypy.serving.request.identity.user_name == None
+        assert cherrypy.serving.request.identity.anonymous
+        assert identity.current.anonymous
+        return 'is_anonymous'
 
-class TestIdentity(unittest.TestCase):
+    @expose(format='json')
+    @identity.require(in_group('peon'))
+    def json(self):
+        return dict(json="restricted_json")
+
+
+class TestIdentity(testutil.TGTest):
 
     def setUp(self):
         # identity requires visit and a failure_url
         test_config = {'global': {
             'visit.on': True, 'identity.on': True,
+            'visit.manager': 'sqlalchemy',
             'identity.failure_url': '/identity_failed',
-            'identity.force_external_redirect': False,
-            'identity.soprovider.encryption_algorithm': None}}
+            'identity.soprovider.encryption_algorithm': None,
+            'identity.http_basic_auth': False,
+            'identity.http_auth_realm': None,
+            'tg.strict_parameters': True,
+            'tg.allow_json': False}}
         original_config = dict()
         for key in test_config['global']:
             original_config[key] = config.get(key)
         self._original_config = original_config
         config.configure_loggers(test_config)
         config.update(test_config['global'])
-        cherrypy.root = IdentityRoot()
-        startup.startTurboGears()
+        self.root = IdentityRoot
+        super(TestIdentity, self).setUp()
         self.init_model()
 
     def tearDown(self):
-        startup.stopTurboGears()
+        super(TestIdentity, self).tearDown()
         config.update(self._original_config)
 
     def init_model(self):
@@ -191,12 +315,12 @@ class TestIdentity(unittest.TestCase):
             user.addTG_Group(other_group)
 
     def test_user_password_parameters(self):
-        """Controller can receive user_name and password parameters."""
-        testutil.create_request('/new_user_setup?user_name=test&password=pw')
-        firstline = cherrypy.response.body[0]
-        assert 'test pw' in firstline, firstline
+        """Test that controller can receive user_name and password parameters."""
+        response = self.app.get('/new_user_setup?user_name=test&password=pw')
+        assert 'test pw' in response, response
 
     def test_user_exists(self):
+        """Test that test user is present in data base."""
         u = TG_User.by_user_name('samIam')
         assert u.email_address == 'samiam@example.com'
 
@@ -213,9 +337,7 @@ class TestIdentity(unittest.TestCase):
     def test_user_password_unicode(self):
         """Test if we can set a non-ascii user password (no encryption)."""
         config.update({'identity.soprovider.encryption_algorithm': None})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = u'garçon'
@@ -227,9 +349,7 @@ class TestIdentity(unittest.TestCase):
     def test_user_password_hashed_sha(self):
         """Test if a sha hashed password is stored in the database."""
         config.update({'identity.soprovider.encryption_algorithm': 'sha1'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = 'password'
@@ -241,9 +361,7 @@ class TestIdentity(unittest.TestCase):
     def test_user_password_hashed_sha_unicode(self):
         """Test if a non-ascii sha hashed password is stored in the database."""
         config.update({'identity.soprovider.encryption_algorithm': 'sha1'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = u'garçon'
@@ -255,9 +373,7 @@ class TestIdentity(unittest.TestCase):
     def test_user_password_hashed_md5(self):
         """Test if a md5 hashed password is stored in the database."""
         config.update({'identity.soprovider.encryption_algorithm': 'md5'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = 'password'
@@ -269,9 +385,7 @@ class TestIdentity(unittest.TestCase):
     def test_user_password_hashed_md5_unicode(self):
         """Test if a non-ascii md5 hashed password is stored in the database."""
         config.update({'identity.soprovider.encryption_algorithm': 'md5'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = u'garçon'
@@ -286,9 +400,7 @@ class TestIdentity(unittest.TestCase):
         test ensures that the encryption algorithm does handle non-unicode
         parameters gracefully."""
         config.update({'identity.soprovider.encryption_algorithm': 'md5'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = u'garçon'.encode('UTF-8')
@@ -301,9 +413,7 @@ class TestIdentity(unittest.TestCase):
         """Test that we can store raw values in the password field
         (without being hashed)."""
         config.update({'identity.soprovider.encryption_algorithm':'sha1'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.set_password_raw('password')
@@ -313,10 +423,9 @@ class TestIdentity(unittest.TestCase):
         hub.end()
 
     def test_user_password_raw_unicode(self):
+        """Test that unicode passwords are encrypted correctly"""
         config.update({'identity.soprovider.encryption_algorithm':'sha1'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.set_password_raw(u'garçon')
@@ -330,9 +439,7 @@ class TestIdentity(unittest.TestCase):
         config.update({'identity.soprovider.encryption_algorithm': 'custom',
             'identity.custom_encryption':
                 'identity.tests.test_identity.mycustomencrypt'})
-        # force new config values to load
-        startup.startTurboGears()
-        testutil.create_request('/')
+        self.app.get('/')
         hub.begin()
         u = TG_User.by_user_name('samIam')
         u.password = 'password'
@@ -343,235 +450,171 @@ class TestIdentity(unittest.TestCase):
 
     def test_anonymous_browsing(self):
         """Test if we can show up anonymously."""
-        testutil.create_request('/')
-        assert identity.current.anonymous
+        response = self.app.get('/is_anonymous')
+        assert 'is_anonymous' in response, response
 
     def test_deny_anonymous(self):
         """Test that we have secured an url from anonymous users."""
-        testutil.create_request('/logged_in_only')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/logged_in_only', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_deny_anonymous_viewable(self):
         """Test that a logged in user can see an resource blocked
         from anonymous users."""
-        testutil.create_request('/logged_in_only?'
+        response = self.app.get('/logged_in_only?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'logged_in_only' in firstline, firstline
+        assert 'logged_in_only' in response, response
 
-    def test_external_redirect(self):
-        """Test that external redirection at identity failures works."""
-        config.update({'identity.failure_url': 'test_login',
-            'identity.force_external_redirect': True})
-        testutil.create_request('/logged_in_only?a=1&b=2')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' not in firstline, firstline
-        path, params = cherrypy.response.headers['Location'].split('?', 1)
-        assert path.endswith('/test_login')
-        params = params.split('&')
-        params.sort()
-        params = '&'.join(params)
-        assert params == 'a=1&b=2&forward_url=%2Flogged_in_only'
-
-    def test_external_redirect_with_funky_params(self):
-        """Test that external redirection properly handles funky parameters."""
-        config.update({'identity.force_external_redirect': True})
-        testutil.create_request('/logged_in_only?'
-            'a-0.x=1&a-0.y=2&a-1.x=3&a-1.y=4')
-        path, params = cherrypy.response.headers['Location'].split('?', 1)
-        assert path.endswith('/identity_failed')
-        params = params.split('&')
-        params.sort()
-        params = '&'.join(params)
-        assert params == ('a-0.x=1&a-0.y=2&a-1.x=3&a-1.y=4'
-            '&forward_url=%2Flogged_in_only')
+    def test_authenticate_header(self):
+        """Test that identity returns correct WWW-Authenticate header."""
+        response = self.app.get('/logged_in_only', status=403)
+        assert 'WWW-Authenticate' not in response.headers
+        config.update({'identity.http_basic_auth': True,
+                       'identity.http_auth_realm': 'test realm'})
+        response = self.app.get('/logged_in_only', status=401)
+        assert response.headers['WWW-Authenticate'] == 'Basic realm="test realm"'
 
     def test_basic_authentication(self):
         """Test HTTP basic authentication mechanism."""
-        credentials = base64.encodestring('samIam:secret')[:-1]
-        testutil.create_request('/logged_in_only', headers={
-            'Authorization': 'Basic %s' % credentials})
-        firstline = cherrypy.response.body[0]
-        assert 'logged_in_only' in firstline, firstline
         credentials = base64.encodestring('samIam')[:-1]
-        testutil.create_request('/logged_in_only', headers={
-            'Authorization': 'Basic %s' % credentials})
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
+        response = self.app.get('/logged_in_only', headers={
+            'Authorization': 'Basic %s' % credentials}, status=403)
+        assert 'identity_failed_answer' in response, response
         credentials = base64.encodestring('samIam:secret:appendix')[:-1]
-        testutil.create_request('/logged_in_only', headers={
+        response = self.app.get('/logged_in_only', headers={
+            'Authorization': 'Basic %s' % credentials}, status=403)
+        assert 'identity_failed_answer' in response, response
+        response = self.app.get('/logged_in_only', headers={
+            'Authorization': 'Basic samIam:secret'}, status=403)
+        assert 'identity_failed_answer' in response, response
+        credentials = base64.encodestring('samIam:secret')[:-1]
+        response = self.app.get('/logged_in_only', headers={
             'Authorization': 'Basic %s' % credentials})
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        testutil.create_request('/logged_in_only', headers={
-            'Authorization': 'Basic samIam:secret'})
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
+        assert 'logged_in_only' in response, response
 
     def test_logout(self):
-        """Test that logout works and session is gets invalid afterwards."""
-        testutil.create_request('/in_peon_group?'
+        """Test that logout works and session id gets invalid afterwards."""
+        response = self.app.get('/in_peon_group?'
             'user_name=samIam&password=secret&login=Login')
-        self.assertEquals("samIam", cherrypy.serving.request.identity.user_name)
-        session_id = re.match("Set-Cookie: (.*?); Path.*",
-            str(cherrypy.response.simple_cookie)).group(1)
-        testutil.create_request('/logout', headers={'Cookie': session_id})
-        self.assertEquals(None, cherrypy.serving.request.identity.user_name)
-        assert cherrypy.serving.request.identity.anonymous
-
-    def test_logout_with_set_identity(self):
-        """Test that logout works even when there is no visit_key
-        (e.g. when testutils.set_identity_user is used)."""
-        request = testutil.DummyRequest()
-        old_user = testutil.test_user
-        user = TG_User.by_user_name("samIam")
-        testutil.set_identity_user(user)
-        testutil.attach_identity(request)
-        testutil.set_identity_user(old_user)
-        testutil.call_with_request(cherrypy.root.logout, request)
-        assert request.identity.anonymous
+        session_id = response.headers['Set-Cookie']
+        response = self.app.get('/logout', headers={'Cookie': session_id})
+        response = self.app.get('/is_anonymous', headers={'Cookie': session_id})
+        assert response.body == 'is_anonymous'
 
     def test_require_group(self):
-        """Test that an anonymous user can not access resource protected by
+        """Test that a anonymous user can not access resource protected by
         require(in_group(...))"""
-        testutil.create_request('/in_peon_group')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/in_peon_group', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_require_expose_required_permission(self):
         """Test that the decorator exposes the correct permissions via _require
         attribute on the actual method."""
-        testutil.create_request('/test_exposed_require')
-        firstline = cherrypy.response.body[0]
-        assert 'require is exposed' in firstline, firstline
+        response = self.app.get('/test_exposed_require')
+        assert 'require is exposed' in response, response
 
     def test_require_group_viewable(self):
         """Test that a user with proper group membership can see a restricted url."""
-        testutil.create_request('/in_peon_group?'
+        response = self.app.get('/in_peon_group?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'in_peon_group' in firstline, firstline
+        assert 'in_peon_group' in response, response
         user = TG_User.by_user_name("samIam")
 
     def test_require_group_viewable(self):
         """Test that the current user and group properties are set correctly."""
-        testutil.create_request('/in_peon_group?'
+        response = self.app.get('/in_peon_group?'
             'user_name=samIam&password=secret&login=Login')
         user = TG_User.by_user_name("samIam")
         group_ids = set((TG_Group.by_group_name("peon").id,
             TG_Group.by_group_name("other").id))
-        assert identity.current.user == user
-        assert identity.current.user_name == user.user_name
-        assert identity.current.user_id == user.id
-        assert identity.current.groups == set(('peon', 'other'))
-        assert identity.current.group_ids == group_ids
 
     def test_user_not_in_right_group(self):
-        """Test that a user is denied access if they aren't in the right group."""
-        testutil.create_request('/in_admin_group?'
-            'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        """Test that a user is denied access if they aren't in the right group.
+        """
+        response = self.app.get('/in_admin_group?'
+            'user_name=samIam&password=secret&login=Login', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_require_permission(self):
-        """Test that an anonymous user is denied access to a permission restricted url."""
-        testutil.create_request('/has_chopper_permission')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        """Test that an anonymous user is denied access to a permission
+        restricted url.
+        """
+        response = self.app.get('/has_chopper_permission', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_require_permission_viewable(self):
         """Test that a user with proper permissions can see a restricted url."""
-        testutil.create_request('/has_chopper_permission?'
+        response = self.app.get('/has_chopper_permission?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'has_chopper_permission' in firstline, firstline
+        assert 'has_chopper_permission' in response, response
 
     def test_user_lacks_permission(self):
-        """Test that a user is denied acces if they don't have the proper permission."""
-        testutil.create_request('/has_boss_permission?'
-            'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        """Test that a user is denied acces if they don't have the proper
+        permission.
+        """
+        response = self.app.get('/has_boss_permission?'
+            'user_name=samIam&password=secret&login=Login', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_user_info_available(self):
         """Test that we can see user information inside our controller."""
-        testutil.create_request('/user_email?'
+        response = self.app.get('/user_email?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'samiam@example.com' in firstline, firstline
+        assert 'samiam@example.com' in response, response
 
     def test_bad_login(self):
         """Test that we are denied access if we provide a bad login."""
-        testutil.create_request('/logged_in_only?'
-            'user_name=samIam&password=wrong&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/logged_in_only?'
+            'user_name=samIam&password=wrong&login=Login', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_restricted_subdirectory(self):
         """Test that we can restrict access to a whole subdirectory."""
-        testutil.create_request('/peon_area/index')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/peon_area/index', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_restricted_subdirectory_viewable(self):
         """Test that we can access a restricted subdirectory
         if we have proper credentials."""
-        testutil.create_request('/peon_area/index?'
+        response = self.app.get('/peon_area/index?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'restricted_index' in firstline, firstline
+        assert 'restricted_index' in response, response
 
     def test_decoratator_in_restricted_subdirectory(self):
         """Test that we can require a different permission
         in a protected subdirectory."""
-        testutil.create_request('/peon_area/in_other_group?'
+        response = self.app.get('/peon_area/in_other_group?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'in_other_group' in firstline, firstline
+        assert 'in_other_group' in response, response
 
     def test_decoratator_failure_in_restricted_subdirectory(self):
         """Test that we can get an identity failure from a decorator
         in a restricted subdirectory"""
-        testutil.create_request('/peon_area/in_admin_group?'
-            'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/peon_area/in_admin_group?'
+            'user_name=samIam&password=secret&login=Login', status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_explicit_checks_in_restricted_subdirectory(self):
         """Test that explicit permission checks in a protected
         directory is handled as expected"""
-        testutil.create_request('/peon_area/in_other_group_explicit_check?'
+        response = self.app.get('/peon_area/in_other_group_explicit_check?'
             'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'in_other_group' in firstline, firstline
+        assert 'in_other_group' in response, response
 
     def test_throwing_identity_exception_in_restricted_subdirectory(self):
         """Test that throwing an IdentityException in a protected
         directory is handled as expected"""
-        testutil.create_request('/peon_area/in_admin_group_explicit_check?'
-            'user_name=samIam&password=secret&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/peon_area/in_admin_group_explicit_check?'
+            'user_name=samIam&password=secret&login=Login', status=403)
+        assert 'identity_failed' in response, response
 
     def test_decode_filter_whenidfails(self):
         """Test that the decode filter doesn't break with nested
         variables and Identity"""
         params = urllib.quote(IdentityRoot._test_encoded_params.decode(
             'utf-8').encode('latin-1'), '=&')
-        testutil.create_request('/test_params?' + params)
-        firstline = cherrypy.response.body[0]
-        assert 'identity_failed_answer' in firstline, firstline
-        assert cherrypy.response.status == "401 Unauthorized"
+        response = self.app.get('/test_params?' + params, status=403)
+        assert 'identity_failed_answer' in response, response
 
     def test_decode_filter_whenidworks(self):
         """Test that the decode filter doesn't break with nested
@@ -579,23 +622,80 @@ class TestIdentity(unittest.TestCase):
         params = urllib.quote(IdentityRoot._test_encoded_params.decode(
             'utf-8').encode('latin-1'), '=&')
         params += '&user_name=samIam&password=secret&login=Login'
-        testutil.create_request('/test_params?' + params)
-        firstline = cherrypy.response.body[0]
-        assert 'params ok' in firstline, firstline
+        response = self.app.get('/test_params?' + params)
+        assert 'params ok' in response, response
 
     def test_user_unicode(self):
         """Test that we can have non-ascii user names."""
         user = TG_User(user_name=u'säm', display_name=u'Sämüel Käsfuß',
             email_address='sam@example.com', password='geheim')
-        testutil.create_request('/logged_in_only?'
+        response = self.app.get('/logged_in_only?'
             'user_name=säm&password=geheim&login=Login')
-        firstline = cherrypy.response.body[0]
-        assert 'logged_in_only' in firstline, firstline
+        assert 'logged_in_only' in response, response
+        self.app.reset()
         credentials = base64.encodestring('säm:geheim')[:-1]
-        testutil.create_request('/logged_in_only', headers={
+        response = self.app.get('/logged_in_only', headers={
             'Authorization': 'Basic %s' % credentials})
-        firstline = cherrypy.response.body[0]
-        assert 'logged_in_only' in firstline, firstline
+        assert 'logged_in_only' in response, response
+
+    def test_json(self):
+        """Test that JSON controllers return the right status codes."""
+        # We check that we get an authorization error, not the server error
+        # caused by the identity_failure controller not accepting JSON.
+        response = self.app.get('/json', status=403)
+        # we get the output of the identity_failure controller in this case
+        assert 'identity_failed_answer' in response, response
+        response = self.app.get('/json?tg_format=json', status=403)
+        # we get the right status code, but not the output of identity_failure
+        assert 'identity_failed_answer' not in response, response
+        response = self.app.get('/peon_area/json', status=403)
+        assert 'identity_failed_answer' in response, response
+        response = self.app.get('/peon_area/json?tg_format=json', status=403)
+        assert 'identity_failed_answer' not in response, response
+        response = self.app.get('/in_peon_group?'
+            'user_name=samIam&password=secret&login=Login')
+        assert 'in_peon_group' in response, response
+        response = self.app.get('/json', status=200)
+        assert 'restricted_json' in response, response
+        response = self.app.get('/json?tg_format=json', status=200)
+        assert 'restricted_json' in response, response
+        response = self.app.get('/peon_area/json', status=200)
+        assert 'restricted_json' in response, response
+        response = self.app.get('/peon_area/json?tg_format=json', status=200)
+        assert 'restricted_json' in response, response
+
+
+    def test_remote_ip(self):
+        """Test that our client IP is detected correctly."""
+        r = self.app.get('/remote_ip', status=200)
+        r = self.app.get('/remote_ip', headers={'Remote-Addr': '127.0.0.1'},
+            status=200)
+        assert r.raw['remote_ip'] == '127.0.0.1'
+        r = self.app.get('/remote_ip',
+            headers={'X-Forwarded-For': '192.168.1.100, 224.50.214.12'},
+            status=200)
+        assert r.raw['remote_ip'] == '224.50.214.12'
+
+    def test_from_localhost(self):
+        """Test we can connect from 127.0.0.1."""
+        r = self.app.get('/from_localhost', headers={'Remote-Addr': '192.168.4.100'},
+            status=403)
+        assert 'identity_failed_answer' in r
+        r = self.app.get('/from_localhost', headers={'Remote-Addr': '127.0.0.1'},
+            status=200)
+        assert 'localhost_only' in r
+
+    def test_from_any_host(self):
+        """Test we can connect from any host in an IP list."""
+        r = self.app.get('/from_any_host', headers={'Remote-Addr': '192.168.4.100'},
+            status=403)
+        assert 'identity_failed_answer' in r
+        r = self.app.get('/from_any_host', headers={'Remote-Addr': '127.0.0.1'},
+            status=200)
+        assert 'hosts_on_list_only' in r
+        r = self.app.get('/from_any_host', headers={'Remote-Addr': '127.0.0.2'},
+            status=200)
+        assert 'hosts_on_list_only' in r
 
 
 class TestTGUser(testutil.DBTest):
@@ -604,23 +704,51 @@ class TestTGUser(testutil.DBTest):
     def setUp(self):
         self._identity_on = config.get('identity.on', False)
         config.update({'identity.on': False})
-        try:
-            self._provider = cherrypy.request.identityProvider
-        except AttributeError:
-            self._provider= None
-        cherrypy.request.identityProvider = None
-        startup.startTurboGears()
-        testutil.DBTest.setUp(self)
+        super(TestTGUser, self).setUp()
+        testutil.start_server()
 
     def tearDown(self):
-        testutil.DBTest.tearDown(self)
-        startup.stopTurboGears()
-        cherrypy.request.identityProvider = self._provider
+        testutil.stop_server()
+        super(TestTGUser, self).tearDown()
         config.update({'identity.on': self._identity_on})
 
     def test_create_user(self):
-        """Check that User can be created outside of a running identity provider."""
+        """Test that User can be created outside of a running identity provider."""
         u = TG_User(user_name='testcase',
             email_address='testcase@example.com',
             display_name='Test Me', password='test')
         assert u.password=='test', u.password
+
+
+class TestIdentityVisitPlugin(unittest.TestCase):
+    def setUp(self):
+        test_config = {'global': {
+            'visit.on': True,
+            'identity.on': True,
+            'identity.source': 'form, http_auth, visit, '}}
+        original_config = dict()
+        for key in test_config['global']:
+            original_config[key] = config.get(key)
+        self._original_config = original_config
+        config.update(test_config['global'])
+
+    def tearDown(self):
+        config.update(self._original_config)
+
+    def test_identity_source(self):
+        """Test that identity.source setting is parsed correctly."""
+        plug = IdentityVisitPlugin()
+        assert plug.identity_from_form in plug.identity_sources
+        assert plug.identity_from_http_auth in plug.identity_sources
+        assert plug.identity_from_visit in plug.identity_sources
+
+    def test_unknown_identity_source(self):
+        """Test that unknown identity source raises identity configuration error."""
+        config.update({'identity.source': 'bogus'})
+        self.assertRaises(IdentityConfigurationException, IdentityVisitPlugin)
+
+    def test_decode_credentials(self):
+        """Test that HTTP basic auth credentials are decoded correctly."""
+        plug = IdentityVisitPlugin()
+        credentials = base64.encodestring('samIam:secret')[:-1]
+        assert plug.decode_basic_credentials(credentials) == ['samIam', 'secret']
