@@ -4,11 +4,6 @@ from math import ceil
 import logging
 import warnings
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
-
 import cherrypy
 try:
     import sqlobject
@@ -57,12 +52,11 @@ class itemwrapper:
         return obj[self.name]
 
 
-def paginate(var_name, default_order='', default_reversed=None, limit=10,
-            max_limit=0, allow_limit_override=None, max_pages=5,
-            max_sort=1000, dynamic_limit=None):
+def paginate(var_name, default_order='', limit=10,
+            max_limit=0, max_pages=5, max_sort=1000, dynamic_limit=None):
     """The famous TurboGears paginate decorator.
 
-    @param var_name: The variable name that the paginate decorator will try
+    @param var_name: the variable name that the paginate decorator will try
     to control. This key must be present in the dictionnary returned from your
     controller in order for the paginate decorator to be able to handle it.
     @type var_name: string
@@ -73,9 +67,6 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
     @type default_order: string or a list of strings. Any string starting with
     "-" (minus sign) indicates a reverse order for that field/column.
 
-    @param default_reversed: Deprecated, use default_order with minus sign.
-    @type default_reversed: Boolean
-
     @param limit: The hard-coded limit that the paginate decorator will impose
     on the number of "var_name" to display at the same time. This value can be
     overridden by the use of the dynamic_limit keyword argument.
@@ -87,9 +78,6 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
     if it is set to None, any change will be allowed.
     @type max_limit: int
 
-    @param allow_limit_override: Deprecated, use max_limit.
-    @type allow_limit_override: Boolean
-
     @param max_pages: Used to generate the tg.paginate.pages variable. If the
     page count is larger than max_pages, tg.paginate.pages will only contain
     the page numbers surrounding the current page at a distance of max_pages/2.
@@ -112,17 +100,6 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
 
     """
 
-    if default_reversed is not None:
-        warnings.warn("default_reversed is deprecated."
-            " Use default_order='-field' to indicate"
-            " default reversed order, or"
-            " default_order=['field1', '-field2, 'field3']"
-            " for multiple fields.", DeprecationWarning, 2)
-    if allow_limit_override is not None:
-        warnings.warn("allow_limit_override is deprecated."
-            " Use max_limit to specify an upper bound for limit.",
-            DeprecationWarning, 2)
-
     def entangle(func):
 
         get = turbogears.config.get
@@ -130,8 +107,7 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
         def decorated(func, *args, **kw):
 
             def kwpop(name, default=None):
-                return kw.pop(var_name + '_tgp_' + name,
-                    kw.pop('tg_paginate_' + name, default))
+                return kw.pop(var_name + '_tgp_' + name, default)
 
             page = kwpop('no')
             if page is None:
@@ -151,8 +127,7 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
 
             try:
                 limit_ = int(kwpop('limit'))
-                if max_limit is not None and not (
-                        allow_limit_override and max_limit == 0):
+                if max_limit is not None:
                     if max_limit <= 0:
                         raise ValueError
                     limit_ = min(limit_, max_limit)
@@ -193,13 +168,7 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
             else:
                 ordering = default_order or []
                 if isinstance(ordering, basestring):
-                    # adapt old style default_order to new style
-                    if default_reversed:
-                        ordering = "-" + ordering
                     ordering = [ordering]
-                elif default_reversed:
-                    raise ValueError("paginate: default_reversed (deprecated)"
-                        " only allowed when default_order is a basestring")
 
             if order:
                 order = str(order)
@@ -246,8 +215,7 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
             input_values =  variable_encode(cherrypy.request.params.copy())
             input_values.pop('self', None)
             for input_key in input_values.keys():
-                if (input_key.startswith(var_name + '_tgp_') or
-                        input_key.startswith('tg_paginate_')):
+                if input_key.startswith(var_name + '_tgp_'):
                     del input_values[input_key]
 
             paginate_instance = Paginate(
@@ -304,11 +272,8 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
 
         if not get('tg.strict_parameters', False):
             # add hint that paginate parameters shall be left intact
-            args = set()
-            for arg in 'no', 'limit', 'order', 'ordering':
-                args.add(var_name + '_tgp_' + arg)
-                args.add('tg_paginate_' + arg)
-            add_tg_args(func, args)
+            add_tg_args(func, (var_name + '_tgp_' + arg
+                for arg in ('no', 'limit', 'order', 'ordering')))
         return decorated
 
     return weak_signature_decorator(entangle)
@@ -433,25 +398,20 @@ def sqlalchemy_get_column(colname, var_data):
     """Return a column from sqlalchemy var_data based on colname."""
     try:
         mapper = var_data.mapper
-    except AttributeError: # SQLAlchemy >= 0.5
+    except AttributeError:
         try:
             mapper = var_data._mapper_zero()
-        except AttributeError: # SQLAlchemy <= 0.4 and SelectResullts
+        except AttributeError:
             mapper = var_data._query.mapper
     propnames = colname.split('.')
-    colname = propnames.pop()
+    colname = propnames.pop()[len(mapper.column_prefix or ''):]
     for propname in propnames:
         prop = mapper.get_property(
             propname, resolve_synonyms=True, raiseerr=False)
         if not prop:
             break
         mapper = prop.mapper
-    if mapper.column_prefix:
-        # for SQLAlchemy <= 0.4.0 we may need to remove the column_prefix
-        return getattr(mapper.c, colname,
-            getattr(mapper.c, colname[len(mapper.column_prefix):], None))
-    else:
-        return getattr(mapper.c, colname, None)
+    return getattr(mapper.c, colname, None)
 
 def sqlobject_get_column(colname, var_data):
     """Return a column from sqlobject var_data based on colname."""
@@ -521,7 +481,7 @@ def sort_data(data, ordering, in_memory=True):
             descending = False
         if order_by:
             col = get_column(order, data)
-            if col is not None:
+            if col:
                 order_cols.append(order_col(col, descending))
                 continue
         if not order_cols:
@@ -560,18 +520,11 @@ def sort_data(data, ordering, in_memory=True):
                             return lambda row: -keys.index(key(row))
                         else:
                             keys.sort()
-                            keys = dict([(k, -n) for n, k in enumerate(keys)])
+                            keys = dict((k, -n) for n, k in enumerate(keys))
                             return lambda row: keys[key(row)]
                 keys = [reverse_key(*key) for key in keys]
                 key = lambda row: [key(row) for key in keys]
-            try:
-                data.sort(key=key, reverse=reverse)
-            except TypeError: # Python 2.3
-                if reverse:
-                    cmpkey = lambda row1, row2: cmp(key(row2), key(row1))
-                else:
-                    cmpkey = lambda row1, row2: cmp(key(row1), key(row2))
-                data.sort(cmpkey)
+            data.sort(key=key, reverse=reverse)
         else:
             log.debug("paginate: sorting in memory not allowed")
     return data
