@@ -1,3 +1,4 @@
+# -*- coding: UTF-8 -*-
 """\
 Front-to-back rapid web development
 ===================================
@@ -28,7 +29,7 @@ problem.
     http://svn.turbogears.org/trunk#egg=turbogears-dev
 """
 
-version = "1.0.10"
+version = "1.1"
 description = "Front-to-back, open-source, rapid web development framework"
 long_description = __doc__
 author = "Kevin Dangoor"
@@ -36,8 +37,7 @@ email = "dangoor+turbogears@gmail.com"
 maintainer = "TurboGears Release Team"
 maintainer_email = "turbogears@googlegroups.com"
 url = "http://www.turbogears.org/"
-download_url = "http://www.turbogears.org/%s/downloads/%s/index" % (
-    '.'.join(version.split('.', 2)[:2]), version)
-dependency_links = [download_url]
-copyright = "Copyright 2005 - 2011 Kevin Dangoor and contributors"
+download_url = "http://www.turbogears.org/download/filelist.html"
+dependency_links = ["http://files.turbogears.org/eggs/"]
+copyright = "Copyright 2005 - 2009 Kevin Dangoor and contributors"
 license = "MIT"
