@@ -6,16 +6,13 @@ import re
 import logging
 import warnings
 import htmlentitydefs
+import socket
+import struct
 from inspect import getargspec, getargvalues
 from itertools import izip, islice, chain
 from operator import isSequenceType
 from urllib import quote, unquote
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
-
 import pkg_resources
 
 from cherrypy import request
@@ -37,19 +34,31 @@ def deprecated(message=None):
             decorate.message = \
                 "Call to deprecated function %s." % func.__name__
         def newFunc(*args, **kwargs):
-            warnings.warn(decorate.message, category=DeprecationWarning,
-                stacklevel = 2)
+            if not decorate.warned:
+                warnings.warn(decorate.message, category=DeprecationWarning,
+                    stacklevel = 2)
+                decorate.warned = True
             return func(*args, **kwargs)
-        try:
-            newFunc.__name__ = func.__name__
-        except TypeError: # not writable in Python 2.3
-            pass
+        newFunc.__name__ = func.__name__
         newFunc.__doc__ = func.__doc__
         newFunc.__dict__.update(func.__dict__)
         return newFunc
     decorate.message = message
+    decorate.warned = False
     return decorate
 
+def missing_dependency_error(name=None):
+    msg = """\
+Before you can run this command, you need to install all the project's
+dependencies by running "python setup.py develop" in the project directory, or
+you can install the application with "python setup.py install", or build an egg
+with "python setup.py bdist_egg" and install it with "easy_install dist/<egg>".
+
+If you are stuck, visit http://docs.turbogears.org/GettingHelp for support."""
+    if name:
+        msg = ("This project requires the %s package but it could not be "
+            "found.\n\n" % name) + msg
+    return msg
 
 class Bunch(dict):
     """Simple but handy collector of a bunch of named stuff."""
@@ -77,10 +86,9 @@ class Bunch(dict):
 
 class DictObj(Bunch):
 
+    @deprecated("Use Bunch instead of DictObj and DictWrapper.")
     def __init__(self, *args, **kw):
         super(DictObj, self).__init__(*args, **kw)
-    __init__ = deprecated(
-        "Use Bunch instead of DictObj and DictWrapper.")(__init__)
 
 DictWrapper = DictObj
 
@@ -250,7 +258,39 @@ def from_kw(func, args, kw, start=0):
 
 
 def adapt_call(func, args, kw, start=0):
-    """Remove excess arguments."""
+    """Remove unsupported func arguments from given args list and kw dict.
+
+    @param func: the callable to inspect for supported arguments
+    @type func: callable
+
+    @param args: the names of the positional arguments intended to be passed
+    to func
+    @type args: list
+
+    @param kw: the keyword arguments intended to be passed to func
+    @type kw: dict
+
+    @keyparam start: the number of items from the start of the argument list of
+    func to disregard. Set start=1 to use adapt_call on a bound method to
+    disregard the implicit self argument.
+
+    @type start: int
+
+    Returns args list and kw dict from which arguments unsupported by func
+    have been removed. The passed in kw dict is also stripped as a side-effect.
+    The returned objects can then be used to call the target function.
+
+    Example:
+
+        def myfunc(arg1, arg2, kwarg1='foo'):
+            pass
+
+        args, kw = adapt_call(myfunc, ['args1, 'bogus1'],
+           {'kwargs1': 'bar', 'bogus2': 'spamm'})
+        # --> ['args1'], {'kwargs1': 'bar'}
+        myfunc(*args, **kw)
+
+    """
     argnames, varargs, kwargs = getargspec(func)[:3]
     del argnames[:start]
     if kwargs in (None, "_decorator__kwargs"):
@@ -427,7 +467,7 @@ def load_class(dottedpath):
         # properly log the exception information and return None
         # to tell caller we did not succeed
         logging.exception('tg.utils: Could not import %s'
-            ' because exception occurred', dottedpath)
+            ' because exception occurred' % dottedpath)
         return None
     return getattr(module, classname)
 
@@ -464,7 +504,7 @@ def parse_http_accept_header(accept):
             item = item.strip()
             if item:
                 items.append((item, q))
-    items.sort(lambda i1, i2: cmp(i2[1], i1[1]))
+    items.sort(key=lambda item: -item[1])
     return [item[0] for item in items]
 
 
@@ -562,7 +602,7 @@ def unquote_cookie(value):
 def get_template_encoding_default(engine_name=None):
     """Return default encoding for template files (Kid, Genshi, etc.)."""
     if engine_name is None:
-        engine_name = config.get('tg.defaultview', 'kid')
+        engine_name = config.get('tg.defaultview', 'genshi')
     return config.get('%s.encoding' % engine_name,
         config.get('%s.default_encoding' % engine_name, 'utf-8'))
 
@@ -674,6 +714,84 @@ def fixentities(htmltext):
     return re.sub("&(\w+);?", repl, htmltext)
 
 
+if hasattr(socket, 'inet_pton') and hasattr(socket, 'AF_INET6'):
+
+    def inet6_aton(addr):
+        """Convert IP6 standard hex notation to IP6 address."""
+        return socket.inet_pton(socket.AF_INET6, addr)
+
+else: # Windows etc.
+
+    import string
+    _inet6_chars = string.hexdigits + ':.'
+
+    def inet6_aton(addr):
+        """Convert IPv6 standard hex notation to IPv6 address.
+
+        Inspired by http://twistedmatrix.com/trac/.
+
+        """
+        faulty = addr.lstrip(_inet6_chars)
+        if faulty:
+            raise ValueError("Illegal character '%c' in IPv6 address" % faulty[0])
+        parts = addr.split(':')
+        elided = parts.count('')
+        extenso = '.' in parts[-1] and 7 or 8
+        if len(parts) > extenso or elided > 3:
+            raise ValueError("Syntactically invalid IPv6 address")
+        if elided == 3:
+            return '\x00' * 16
+        if elided:
+            zeros = ['0'] * (extenso - len(parts) + elided)
+            if addr.startswith('::'):
+                parts[:2] = zeros
+            elif addr.endswith('::'):
+                parts[-2:] = zeros
+            else:
+                idx = parts.index('')
+                parts[idx:idx+1] = zeros
+        if len(parts) != extenso:
+            raise ValueError("Syntactically invalid IPv6 address")
+        if extenso == 7:
+            ipv4 = parts.pop()
+            if ipv4.count('.') != 3:
+                raise ValueError("Syntactically invalid IPv6 address")
+            parts = [int(x, 16) for x in parts]
+            return struct.pack('!6H', *parts) + socket.inet_aton(ipv4)
+        else:
+            parts = [int(x, 16) for x in parts]
+            return struct.pack('!8H', *parts)
+
+
+def inet_aton(addr):
+    """Convert IPv4 or IPv6 notation to IPv6 address."""
+    if ':' in addr:
+        return inet6_aton(addr)
+    else:
+        return struct.pack('!QL', 0, 0xffff) + socket.inet_aton(addr)
+
+
+def _inet_prefix(addr, masked):
+    """Remove the number of masked bits from the IPV6 address."""
+    hi, lo = struct.unpack("!QQ", addr)
+    return (hi << 64 | lo) >> masked
+
+
+def match_ip(cidr, ip):
+    """Check whether IP address matches CIDR IP address block."""
+    if '/' in cidr:
+        cidr, prefix = cidr.split('/', 1)
+        masked = (':' in cidr and 128 or 32) - int(prefix)
+    else:
+        masked = None
+    cidr = inet_aton(cidr)
+    ip = inet_aton(ip)
+    if masked:
+        cidr = _inet_prefix(cidr, masked)
+        ip = _inet_prefix(ip, masked)
+    return ip == cidr
+
+
 __all__ = ["Bunch", "DictObj", "DictWrapper", "Enum", "setlike",
            "get_package_name", "get_model", "load_project_config",
            "ensure_sequence", "has_arg", "to_kw", "from_kw", "adapt_call",
@@ -685,4 +803,4 @@ __all__ = ["Bunch", "DictObj", "DictWrapper", "Enum", "setlike",
            "to_unicode", "to_utf8", "quote_cookie", "unquote_cookie",
            "get_template_encoding_default", "get_mime_type_for_format",
            "mime_type_has_charset", "find_precision", "copy_if_mutable",
-           "deprecated"]
+           "match_ip", "deprecated"]
