@@ -3,11 +3,6 @@ import sha
 import threading
 import time
 
-try:
-    set()
-except NameError:
-    from sets import Set as set
-
 from random import random
 from datetime import timedelta, datetime
 
@@ -16,6 +11,7 @@ import pkg_resources
 
 from cherrypy.filters.basefilter import BaseFilter
 from turbogears import config
+from turbogears.util import load_class
 
 log = logging.getLogger("turbogears.visit")
 
@@ -36,14 +32,27 @@ def set_current(visit):
 
 def _create_visit_manager(timeout):
     """Create a VisitManager based on the plugin specified in the config file."""
-    plugin_name = config.get("visit.manager", "sqlobject")
+    plugin_name = config.get("visit.manager", "sqlalchemy")
     plugins = pkg_resources.iter_entry_points(
         "turbogears.visit.manager", plugin_name)
     log.debug("Loading visit manager from plugin: %s", plugin_name)
+    provider_class = None
     for entrypoint in plugins:
-        plugin = entrypoint.load()
-        return plugin(timeout)
-    raise RuntimeError("VisitManager plugin missing: %s" % plugin_name)
+        try:
+            provider_class = entrypoint.load()
+            break
+        except ImportError, e:
+            log.error("Error loading visit plugin '%s': %s", entrypoint, e)
+
+    if not provider_class and '.' in plugin_name:
+        try:
+            provider_class = load_class(plugin_name)
+        except ImportError, e:
+            log.error("Error loading visit class '%s': %s", plugin_name, e)
+    if not provider_class:
+        raise RuntimeError("VisitManager plugin missing: %s" % plugin_name)
+    return provider_class(timeout)
+
 
 class MonkeyDecodingFilter(BaseFilter):
 
@@ -108,6 +117,7 @@ class MonkeyDecodingFilter(BaseFilter):
 def start_extension():
     # TODO: This should alway be active even when visit is not on,
     # we should find a better place to load this code...
+    global _manager
 
     # Monkey patch CP Decoding filter
     # TODO: Is there a better way to inject this?
@@ -126,12 +136,13 @@ def start_extension():
     # Bail out if the application hasn't enabled this extension
     if not config.get("visit.on", False):
         return
+
     # Bail out if this extension is already running
-    global _manager
     if _manager:
+        log.warning("Visit manager already running.")
         return
 
-    log.info("Visit Tracking starting")
+    log.info("Visit Tracking starting.")
     # How long may the visit be idle before a new visit ID is assigned?
     # The default is 20 minutes.
     timeout = timedelta(minutes=config.get("visit.timeout", 20))
@@ -142,7 +153,8 @@ def start_extension():
     # Install Filter into the root filter chain
     if not hasattr(cherrypy.root, "_cp_filters"):
         cherrypy.root._cp_filters = list()
-    cherrypy.root._cp_filters.append(visit_filter)
+    if not visit_filter in cherrypy.root._cp_filters:
+        cherrypy.root._cp_filters.append(visit_filter)
 
 def shutdown_extension():
     # Bail out if this extension is not running.
@@ -178,7 +190,6 @@ class VisitFilter(BaseFilter):
     """A filter that automatically tracks visitors."""
 
     def __init__(self):
-        log.info("Visit filter initialized")
         get = config.get
         # Where to look for the session key in the request and in which order
         self.source = [s.strip().lower() for s in
@@ -188,7 +199,7 @@ class VisitFilter(BaseFilter):
         # Get the name to use for the identity cookie.
         self.cookie_name = get("visit.cookie.name", "tg-visit")
         # and the name of the request param. MUST NOT contain dashes or dots,
-        # otherwise the NestedVariablesFilter will chocke on it.
+        # otherwise the NestedVariablesFilter will choke on it.
         self.visit_key_param = get("visit.form.name", "tg_visit")
         # TODO: The path should probably default to whatever
         # the root is masquerading as in the event of a
@@ -203,6 +214,7 @@ class VisitFilter(BaseFilter):
         # Use max age only if the cookie shall explicitly be permanent
         self.cookie_max_age = get("visit.cookie.permanent",
             False) and int(get("visit.timeout", "20")) * 60 or None
+        log.info("Visit filter initialised")
 
     def before_main(self):
         """Check whether submitted request belongs to an existing visit."""
@@ -228,12 +240,12 @@ class VisitFilter(BaseFilter):
                 if visit_key:
                     visit = _manager.visit_for_key(visit_key)
                     break
-            if not visit:
+            if visit:
+                log.debug("Using visit from request with key: %s", visit_key)
+            else:
                 visit_key = self._generate_key()
                 visit = _manager.new_visit_with_key(visit_key)
                 log.debug("Created new visit with key: %s", visit_key)
-            else:
-                log.debug("Using visit from request with key: %s", visit_key)
             self.send_cookie(visit_key)
             set_current(visit)
         # Inform all the plugins that a request has been made for the current
@@ -280,7 +292,7 @@ class VisitFilter(BaseFilter):
         max_age = self.cookie_max_age
         if max_age:
             # use 'expires' because MSIE ignores 'max-age'
-            cookies[self.cookie_name]['expires'] = time.strftime(
+            cookies[self.cookie_name]['expires'] = '"%s"' % time.strftime(
                 "%a, %d-%b-%Y %H:%M:%S GMT",
                 time.gmtime(time.time() + max_age))
             # 'max-age' takes precedence on standard conformant browsers
@@ -298,11 +310,10 @@ class BaseVisitManager(threading.Thread):
         self.queue = dict()
         self.lock = threading.Lock()
         self._shutdown = threading.Event()
-        self.interval = 30
-        self.setDaemon(True)
-        # We need to create the visit model before the manager thread is
-        # started.
+        self.interval = config.get('visit.interval', 30) # seconds
+        # We must create the visit model before the manager thread is started.
         self.create_model()
+        self.setDaemon(True)
         self.start()
 
     def create_model(self):
@@ -332,22 +343,29 @@ class BaseVisitManager(threading.Thread):
             self.lock.release()
 
     def shutdown(self, timeout=None):
-        self._shutdown.set()
-        self.join(timeout)
+        try:
+            self.lock.acquire()
+            self._shutdown.set()
+            self.join(timeout)
+        finally:
+            self.lock.release()
         if self.isAlive():
             log.error("Visit Manager thread failed to shutdown.")
 
     def run(self):
         while not self._shutdown.isSet():
             self.lock.acquire()
+            if self._shutdown.isSet():
+                self.lock.release()
+                continue
             queue = None
             try:
                 # make a copy of the queue and empty the original
                 if self.queue:
                     queue = self.queue.copy()
                     self.queue.clear()
+                if queue is not None:
+                    self.update_queued_visits(queue)
             finally:
                 self.lock.release()
-            if queue is not None:
-                self.update_queued_visits(queue)
             self._shutdown.wait(self.interval)
