@@ -2,31 +2,29 @@ import os
 import itertools
 import pkg_resources
 import warnings
-from turbogears import view, validators, startup, config
-from turbogears.util import setlike, to_unicode, copy_if_mutable, get_package_name
+from turbogears import view, startup, config
+from turbogears.util import setlike, to_unicode, copy_if_mutable, \
+    get_package_name, request_available
 from turbogears.i18n.utils import get_locale
 from turbogears.widgets.meta import MetaWidget, load_kid_template
-from cherrypy.config import configs
+from cherrypy import request
+import tgmochikit
 
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-__all__ = ["load_widgets", "all_widgets", "Widget", "CompoundWidget",
-           "WidgetsList", "register_static_directory",
-           "static", "Resource", "Link", "CSSLink", "JSLink",
-           "Source", "CSSSource", "JSSource", "js_location", "mochikit", "jsi18nwidget",
-           "WidgetDescription", "set_with_self"]
+__all__ = ['load_widgets', 'all_widgets', 'Widget', 'CompoundWidget',
+    'WidgetsList', 'register_static_directory', 'static',
+    'Resource', 'Link', 'CSSLink', 'JSLink',
+    'Source', 'CSSSource', 'JSSource',
+    'js_location', 'mochikit', 'jsi18nwidget',
+    'WidgetDescription', 'set_with_self']
 
 
 class Enum(set):
-    """Enum used at js_locations which is less strict than
-    ``turbogears.utils.Enum`` and serves our purposes as well as allowing
-    any object with ``retrieve_javascript``, ``retrieve_css``, and
-    ``location`` attributes to provide resources to the template when
-    scanned in ``turbogears.controllers._process_output``.
+    """Enum used at js_locations.
 
+    This is less strict than ``turbogears.utils.Enum`` and serves our
+    purposes as well as allowing any object with ``retrieve_javascript``,
+    ``retrieve_css``, and ``location`` attributes to provide resources to
+    the template when scanned in ``turbogears.controllers._process_output``.
 
     Example:
 
@@ -70,19 +68,16 @@ def load_widgets():
 all_widgets = set()
 
 PlainHTML = load_kid_template("""
-                              <html xmlns:py="http://purl.org/kid/ns#"
-                                  py:replace="elements"
-                              />
-                              """,
-                              modname="turbogears.widgets.plainhtml"
-                             )[0]
+    <html xmlns:py="http://purl.org/kid/ns#" py:replace="elements"/>
+""", modname='turbogears.widgets.plainhtml')[0]
+
 
 #############################################################################
 # Widgets base classes                                                      #
 #############################################################################
+
 class Widget(object):
-    """
-    A TurboGears Widget.
+    """A TurboGears Widget.
 
    '__init__' and 'update_params' are the only methods you might need to
    care extending.
@@ -127,6 +122,7 @@ class Widget(object):
    4) A callable can be passed and it will be called automatically when sending
       variables to the template. This can be handy to pick up parameters which
       change in every request or affect many widgets simultaneously.
+
     """
 
     __metaclass__ = MetaWidget
@@ -140,10 +136,11 @@ class Widget(object):
     params_doc = {}
 
     def __init__(self, name=None, template=None, default=None, **params):
-        """
-        All initialization has to take place in this method. It's not
-        thread-safe to mutate widget's attributes outside this method or
-        anytime after widget's first display.
+        """Widget initialization.
+
+        All initialization has to take place in this method.
+        It's not thread-safe to mutate widget's attributes outside this method
+        or anytime after widget's first display.
 
         *Must* call super(MyWidget, self).__init__(*args, **kw) cooperatively,
         unless, of course, your know what you're doing. Preferably this should
@@ -169,19 +166,24 @@ class Widget(object):
         "params". Some widgets accept parameters at the constructor which are
         not listed params, these parameter won't be passed to the template, be
         automatically called, etc..
+
         """
         self._declaration_counter = counter.next()
         if name:
             self.name = name
+
         if template:
-            (self.template_c, self.template) = load_kid_template(template)
-        if default:
+            self.template_c, self.template = load_kid_template(template)
+
+        if default is not None:
             self.default = default
+
         # logic for managing the params attribute
         for param in self.__class__.params:
             if param in params:
                 # make sure we don't keep references to mutables
                 setattr(self, param, copy_if_mutable(params.pop(param)))
+
             else:
                 if hasattr(self, param):
                     # make sure we don't alter mutable class attributes
@@ -192,21 +194,22 @@ class Widget(object):
                         setattr(self, param, value[0])
                 else:
                     setattr(self, param, None)
+
         for unused in params.iterkeys():
             warnings.warn('keyword argument "%s" is unused at %r instance' % (
                 unused, self.__class__.__name__))
 
     def adjust_value(self, value, **params):
-        """
-        Adjusts the value sent to the template on display.
-        """
+        """Adjust the value sent to the template on display."""
         if value is None:
-            return self.default
-        else:
-            return value
+            value = self.default
+            if callable(value):
+                value = value()
+        return value
 
     def update_params(self, params):
-        """
+        """Update the template parameters.
+
         This method will have the last chance to update the variables sent to
         the template for the specific request. All parameters listed at class
         attribute 'params' will be available at the 'params' dict this method
@@ -215,6 +218,7 @@ class Widget(object):
         *Must* call super(MyWidget, self).update_params(params) cooperatively,
         unless, of course, your know what you're doing. Preferably this should
         be done before any actual work is done in the method.
+
         """
         pass
 
@@ -222,76 +226,87 @@ class Widget(object):
     # there is rarely the need to override or extend them if inheritting
     # directly or indirectly from Widget.
 
-    # update_data has been deprecated
-    update_data = update_params
-
     def __call__(self, *args, **params):
-        """
-        Delegate to display. Used as an alias to avoid tiresome typing
-        """
+        """Delegate to display. Used as an alias to avoid tiresome typing."""
         return self.display(*args, **params)
 
     def display(self, value=None, **params):
-        """
-        Display the widget in a Kid template. Returns an elementtree node
-        instance. If you need serialized output in a string call 'render'
-        instead.
-        Probably you will not need to override or extend if inhertitting from
+        """Display the widget in a Kid template.
+
+        Returns an elementtree node instance. If you need serialized output
+        in a string, call 'render' instead.
+
+        Probably you will not need to override or extend if inhertiting from
         Widget.
 
         @params:
 
-        value    : The value to display in the widget.
-        **params : Extra parameters specific to the widget. All keyword params
-                   supplied will pass through the update_params method which will
-                   have a last chance to modify them before reaching the
-                   template.
+        value   : The value to display in the widget.
+        **params: Extra parameters specific to the widget. All keyword params
+                  supplied will pass through the update_params method which
+                  will have a last chance to modify them before reaching the
+                  template.
+
         """
 
         if not getattr(self, 'template_c', False):
             warnings.warn("Widget instance '%r' has no template defined" % self)
             return None
+
         # logic for managing the params attribute
         for param in self.__class__.params:
             if param in params:
                 param_value = params[param]
                 if callable(param_value):
                     param_value = param_value()
+
             else:
                 # if the param hasn't been overridden (passed as a keyword
                 # argument inside **params) put the corresponding instance
                 # value inside params.
                 param_value = getattr(self, param, None)
+
             # make sure we don't pass a reference to mutables
             params[param] = copy_if_mutable(param_value)
-        params["name"] = self.name
-        params["value"] = to_unicode(self.adjust_value(value, **params))
+
+        if not params.get('name'):
+            params['name'] = self.name
+        params['value'] = to_unicode(self.adjust_value(value, **params))
         self.update_params(params)
-        # update_data has been deprecated
-        self.update_data(params)
+
         try:
             transform = view.engines['kid'].transform
+
         except (KeyError, AttributeError):
             # this can happen if you render a widget before application startup
             # when view.load_engines() has not yet been called
             raise RuntimeError("Trying to render a widget,"
                 " but Kid templating engine is not yet loaded.")
-        return transform(params, self.template_c)
+
+        output = transform(params, self.template_c)
+        if request_available() and request.tg_template_enginename == 'genshi' \
+                and len(getattr(request, 'tg_widgets_path', [])) == 1:
+            import genshi
+            output = genshi.input.ET(output)
+        return output
 
     def render(self, value=None, format="html", **params):
-        """
-        Exactly the same as display() but return serialized output instead.
-        Useful for debugging or to display the widget in a non-Kid template like
-        Cheetah, STAN, ...
+        """Exactly the same as display() but return serialized output instead.
+
+        Useful for debugging or to display the widget in a non-Kid template
+        like Cheetah, STAN, ...
+
         """
         elem = self.display(value, **params)
         t = PlainHTML(elements=elem)
         return t.serialize(output=format, fragment=True)
 
     def retrieve_javascript(self):
-        """
-        Return a setlike instance with all the JSLinks and JSSources the
-        widget needs.
+        """Return the needed JavaScript ressources.
+
+        Return a setlike instance containing all the JSLinks and JSSources
+        the widget needs.
+
         """
         scripts = setlike()
         for script in self.javascript:
@@ -299,9 +314,11 @@ class Widget(object):
         return scripts
 
     def retrieve_css(self):
-        """
-        Return a setlike instance with all the CSSLinks and CSSSources the
-        widget needs.
+        """Return the needed CSS ressources.
+
+        Return a setlike instance with all the CSSLinks and CSSSources
+        the widget needs.
+
         """
         css = setlike()
         for cssitem in self.css:
@@ -309,9 +326,7 @@ class Widget(object):
         return css
 
     def _get_is_named(self):
-        """
-        Return True if the widget has overrided it's default name, else False.
-        """
+        """Return True if the widget has overridden its default name."""
         if self.name != "widget":
             return True
         else:
@@ -330,12 +345,14 @@ class Widget(object):
             ["%s=%r" % (var, getattr(self, var))
                 for var in ['name'] + self.__class__.params]))
 
+
 class CompoundWidget(Widget):
     """A widget that can contain other widgets.
 
-    A compund widget is a widget that can group several widgets to make a
-    complex widget. Child widget names must be listed at ther widget's
+    A compound widget is a widget that can group several widgets to make a
+    complex widget. Child widget names must be listed at their widget's
     ``member_widgets`` attribute.
+
     """
     compound = True
     member_widgets = []
@@ -350,7 +367,7 @@ class CompoundWidget(Widget):
         super(CompoundWidget, self).__init__(*args, **kw)
 
     def iter_member_widgets(self):
-        """Iteratetes over all the widget's children"""
+        """Iterates over all the widget's children"""
         for member in self.__class__.member_widgets:
             attr = getattr(self, member, None)
             if isinstance(attr, list):
@@ -373,9 +390,11 @@ class CompoundWidget(Widget):
         d['params_for'] = lambda f: self.params_for(f, **widgets_params)
 
     def value_for(self, item, value):
-        """
+        """Get value for member widget.
+
         Pick up the value for a given member_widget 'item' from the
         value dict passed to this widget.
+
         """
         name = getattr(item, "name", item)
         if isinstance(value, dict):
@@ -384,22 +403,26 @@ class CompoundWidget(Widget):
             return None
 
     def params_for(self, item, **params):
-        """
+        """Get params for member widget.
+
         Pick up the params for the given member_widget 'item' from
         the params dict passed to this widget.
+
         """
         name = getattr(item, "name", item)
         item_params = {}
-        for k,v in params.iteritems():
+        for k, v in params.iteritems():
             if isinstance(v, dict):
                 if name in v:
                     item_params[k] = v[name]
         return item_params
 
     def retrieve_javascript(self):
-        """
-        Retrieve the javascript for all the member widgets and
+        """Get JavaScript for the member widgets.
+
+        Retrieve the JavaScript for all the member widgets and
         get an ordered union of them.
+
         """
         scripts = setlike()
         for script in self.javascript:
@@ -410,9 +433,11 @@ class CompoundWidget(Widget):
         return scripts
 
     def retrieve_css(self):
-        """
-        Retrieve the css for all the member widgets and
+        """Get CSS for the member widgets.
+
+        Retrieve the CSS for all the member widgets and
         get an ordered union of them.
+
         """
         css = setlike()
         for cssitem in self.css:
@@ -426,81 +451,110 @@ class CompoundWidget(Widget):
 #############################################################################
 # Declarative widgets support                                               #
 #############################################################################
+
 class MetaWidgetsList(type):
-    """
-    Metaclass for WidgetLists. Takes care that the resulting WidgetList
-    has all widgets in the same order as they were declared.
+    """Metaclass for WidgetLists.
+
+    Takes care that the resulting WidgetList has all widgets in the same order
+    as they were declared.
+
     """
     def __new__(meta, class_name, bases, class_dict):
         declared_widgets = []
+        for base in bases:
+            declared_widgets.extend(getattr(base, 'declared_widgets', []))
         for name, value in class_dict.items():
             if isinstance(value, Widget):
                 if not value.is_named:
                     value.name = name
                 declared_widgets.append(value)
                 del class_dict[name]
-        declared_widgets.sort(lambda a, b: cmp(a._declaration_counter,
-                                               b._declaration_counter))
+        declared_widgets.sort(key=lambda w: w._declaration_counter)
         cls = type.__new__(meta, class_name, bases, class_dict)
         cls.declared_widgets = declared_widgets
         return cls
 
+
 class WidgetsList(list):
-    """
-    A widget list. That's really all. A plain old list that you can
-    declare as a classs with widgets ordered as attributes. Syntactic
-    sugar for an unsweet world.
+    """A widget list.
+
+    That's really all. A plain old list that you can declare as a classs
+    with widgets ordered as attributes. Syntactic sugar for an unsweet world.
+
     """
     __metaclass__ = MetaWidgetsList
 
-    def __init__(self):
-        super(WidgetsList, self).__init__()
-        self.extend(self.declared_widgets)
-        if len(self) == 0:
+    def __init__(self, *args):
+        super(WidgetsList, self).__init__(self.declared_widgets)
+        if args:
+            if len(args) == 1:
+                args = args[0]
+                if isinstance(args, Widget):
+                    args = [args]
+            self.extend(args)
+        if not self:
             warnings.warn("You have declared an empty WidgetsList")
 
+    # Make WidgetsList instances usable as Widget containers in the dict
+    # returned by controller methods, so that the JS/CSS resources of all
+    # contained widgets get picked up by the templates:
+
+    def retrieve_javascript(self):
+        return itertools.chain(*(w.retrieve_javascript() for w in self
+            if callable(getattr(w, 'retrieve_javascript', None))))
+
+    def retrieve_css(self):
+        return itertools.chain(*(w.retrieve_css() for w in self
+            if callable(getattr(w, 'retrieve_css', None))))
+
+
 #############################################################################
 # CSS, JS and mochikit stuff                                                #
 #############################################################################
+
 def register_static_directory(modulename, directory):
-    """
-    Sets up a static directory for JavaScript and css files. You can refer
-    to this static directory in templates as ${tg.widgets}/modulename
+    """Set up a static directory for JavaScript and CSS files.
+
+    You can refer to this static directory in templates as
+    ${tg.widgets}/modulename
+
     """
     directory = os.path.abspath(directory)
-    config.update({'/tg_widgets/%s' % modulename :
-                   {
-                       'static_filter.on' : True,
-                       'static_filter.dir' : directory
-                   }
-                  }
-                 )
+    config.update({'/tg_widgets/%s' % modulename: {
+        'static_filter.on' : True,
+        'static_filter.dir': directory
+    }})
 
 static = "turbogears.widgets"
 
 register_static_directory(static,
-                          pkg_resources.resource_filename(__name__,
-                                                          "static")
-                         )
+    pkg_resources.resource_filename(__name__, 'static'))
+
+register_static_directory('turbogears',
+    pkg_resources.resource_filename('turbogears', 'static'))
 
-register_static_directory("turbogears",
-                          pkg_resources.resource_filename("turbogears",
-                                                          "static")
-                         )
 
 def set_with_self(self):
     theset = setlike()
     theset.add(self)
     return theset
 
+
 class Resource(Widget):
+    """A resource for your widget.
+
+    For example, this can be a link to an external JavaScript/CSS file
+    or inline source to include at the template the widget is displayed.
+
     """
-    A resource for your widget, like a link to external JS/CSS or inline
-    source to include at the template the widget is displayed.
-    """
-    pass
+    order = 0
+    params = ['order']
+    params_doc = {'order': "JS and CSS are sorted in this 'order' at render time"}
+
 
 class Link(Resource):
+    """A widget resource that is a link."""
+
     def __init__(self, mod, *args, **kw):
         super(Link, self).__init__(*args, **kw)
         self.mod = mod
@@ -508,8 +562,7 @@ class Link(Resource):
     def update_params(self, d):
         super(Link, self).update_params(d)
         d["link"] = "/%stg_widgets/%s/%s" % (startup.webpath,
-                                             self.mod,
-                                             self.name)
+            self.mod, self.name)
 
     def __hash__(self):
         return hash(self.mod + self.name)
@@ -520,6 +573,8 @@ class Link(Resource):
 
 
 class CSSLink(Link):
+    """A CSS links."""
+
     template = """
     <link rel="stylesheet"
         type="text/css"
@@ -533,9 +588,13 @@ class CSSLink(Link):
 
     retrieve_css = set_with_self
 
+
 js_location = Enum('head', 'bodytop', 'bodybottom')
 
+
 class JSLink(Link):
+    """A JavaScript link."""
+
     template = """
     <script type="text/javascript" src="$link"
         charset="$charset" defer="${defer and 'defer' or None}"/>
@@ -552,53 +611,58 @@ class JSLink(Link):
         location = kw.pop('location', None)
         super(JSLink, self).__init__(*args, **kw)
         if location:
-            if location not in js_location:
-                raise ValueError, "JSLink location should be in %s" % js_location
+            #if location not in js_location:
+            #    raise ValueError, "JSLink location should be in %s" % js_location
             self.location = location
 
     retrieve_javascript = set_with_self
 
-class MochiKitLink(JSLink):
-    """
-    This is a specialized JSLink to allow for suppression of MochiKit-inclusion
-    by core TG.
 
-    The rationale for this is that MK it version 1.3.1 has some issues that are
-    remedied by MK 1.4, but the latter is unfortunately unrleased.
+class TGMochiKit(JSLink):
+    """This JSLink includes MochkKit by means of the tgMochiKit project.
 
-    To remedy this, one can include MK 1.4 by oneself (using e.g. tgMochiKit) - but
-    this will conflict with the contained version which is made use of in several
-    TG widgets.
+    Depending on three config options, you can decide
 
-    By setting the configuration option
+    - tg_mochikit.version (default: 1.3.1) - which version to use
+    - tg_mochikit.packed  (default: False) - deliver the JS packed or unpacked
+    - tg_mochikit.xhtml   (default: False) - be XHTML-compliant or not in case you deliver unpacked
 
-      tg.mochikit_suppress = True
+    For an explanation of the options, see the tgMochiKit documentation at
 
-    the incusion of MK via built-in widgets will be suppressed. Of course then one
-    has to include the proper version explicitly, or via tg.include_widgets.
+        http://docs.turbogears.org/tgMochiKit
 
-    If 'turbogears.mochikit' is part of tg.include_widgets, a warning is issued.
     """
 
-    template = """
-    <script xmlns:py="http://purl.org/kid/ns#" py:if="not suppress" type="text/javascript" src="$link"/>
+    template = """<script xmlns:py="http://purl.org/kid/ns#"
+    py:for="js in javascripts" py:replace="js.display()"
+    />
     """
 
-    def update_params(self, p):
-        super(MochiKitLink, self).update_params(p)
-        suppress = config.get('tg.mochikit_suppress', False)
-        p['suppress'] = suppress
+    def retrieve_javascript(self):
+        tgmochikit.init(register_static_directory, config)
+        if config.get('tg.mochikit_suppress', False):
+            if 'turbogears.mochikit' in config.get('tg.include_widgets', []):
+                warnings.warn("""\
+tg.mochikit_suppress == True, but 'turbogears.mochikit' is to be included via
+'tg.include_widgets'. This is a contradiction, and mochikit_suppress overrides
+the inclusion to prevent subtle errors due to version mixing of MochiKit.""")
+            return []
+        javascripts = [JSLink("tgmochikit", path)
+            for path in tgmochikit.get_paths()]
+        return javascripts
 
-        if suppress and 'turbogears.mochikit' in config.get('tg.include_widgets', []):
-            warnings.warn("""tg.mochikit_suppress == True, but 'turbogears.mochikit' is to be included via 'tg.mochikit_suppress'.
-This is a contradiction, and mochikit_suppress overrides the inclusion to prevent subtle errors due to version mixing of MochiKit.""")
+    def update_params(self, d):
+        super(TGMochiKit, self).update_params(d)
+        d['javascripts'] = self.retrieve_javascript()
+
+mochikit = TGMochiKit("tgmochikit")
 
-mochikit = MochiKitLink("turbogears", "js/MochiKit.js")
 
 class JSI18NWidget(Widget):
-    """
+    """JavaScript i18n support widget.
+
     This is a widget that can be used to add support for
-    i18n of javascript texts.
+    internationalization of JavaScript texts.
 
     It will basically add an implementation of the
     _ function that then can be used to wrap text literals
@@ -612,9 +676,10 @@ class JSI18NWidget(Widget):
     tg.include_widgets = ['turbogears.jsi18nwidget']
 
     in your application configuration.
+
     """
 
-    parameters = ['locale_catalog_providers']
+    params = ['locale_catalog_providers']
 
     locale_catalog_providers = []
 
@@ -622,8 +687,11 @@ class JSI18NWidget(Widget):
         super(JSI18NWidget, self).__init__(*args, **kwargs)
         self._initialized = False
 
-    def register_package_provider(self, package=None, directory=None, domain=None):
+    def register_package_provider(self,
+            package=None, directory=None, domain=None):
+
         parent = self
+
         class PackageProvider(object):
             def __init__(self):
                 self._initialized = False
@@ -656,6 +724,7 @@ class JSI18NWidget(Widget):
                         js.append(link)
                         break
                 return js
+
         self.locale_catalog_providers.append(PackageProvider())
 
     def retrieve_javascript(self):
@@ -672,7 +741,7 @@ class JSI18NWidget(Widget):
 
     def linked_file_exists(self, widget):
         key = '/tg_widgets/%s' % widget.mod
-        wp = configs.get(key)
+        wp = config.get(key)
         if wp:
             dir = wp.get('static_filter.dir')
             if dir:
@@ -682,6 +751,7 @@ class JSI18NWidget(Widget):
 
 jsi18nwidget = JSI18NWidget()
 
+
 class Source(Resource):
     params = ["src"]
 
@@ -695,7 +765,10 @@ class Source(Resource):
     def __eq__(self, other):
         return self.src == getattr(other, "src", None)
 
+
 class CSSSource(Source):
+    """A CSS source snippet."""
+
     template = """
     <style type="text/css" media="$media">$src</style>
     """
@@ -706,7 +779,10 @@ class CSSSource(Source):
 
     retrieve_css = set_with_self
 
+
 class JSSource(Source):
+    """A JavaScript source snippet."""
+
     template = """
     <script type="text/javascript"
         defer="${defer and 'defer' or None}">$src</script>
@@ -726,20 +802,22 @@ class JSSource(Source):
 
     retrieve_javascript = set_with_self
 
+
 #############################################################################
 # Classes for supporting the toolbox widget browser                         #
 #############################################################################
+
 just_the_widget = load_kid_template("""
-                                    <div xmlns:py="http://purl.org/kid/ns#"
-                                      py:content="for_widget.display()"
-                                    />
-                                    """
-                                   )[0]
+    <div xmlns:py="http://purl.org/kid/ns#" py:content="for_widget.display()"/>
+""")[0]
+
 
 class MetaDescription(MetaWidget):
-    """
-    Metaclass for widget descriptions. Makes sure the widget browser knows
-    about all of them as soon as they come into existence.
+    """Metaclass for widget descriptions.
+
+    Makes sure the widget browser knows about all of them as soon as they
+    come into existence.
+
     """
     def __init__(cls, name, bases, dct):
         super(MetaDescription, cls).__init__(name, bases, dct)
@@ -747,12 +825,14 @@ class MetaDescription(MetaWidget):
         if name != "WidgetDescription" and register:
             all_widgets.add(cls)
 
+
 class WidgetDescription(CompoundWidget):
-    """
-    A description for a Widget. Make's the 'for_widget' widget appear in the
-    browser.
-    It's a nice way to show off to your friends your coolest new widgets and
-    to have a testing platform while developing them.
+    """A description for a Widget.
+
+    Makes the 'for_widget' widget appear in the browser. It's a nice way to
+    show off to your friends your coolest new widgets and to have a testing
+    platform while developing them.
+
     """
     __metaclass__ = MetaDescription
 
@@ -789,6 +869,7 @@ class WidgetDescription(CompoundWidget):
     def retrieve_javascript(self):
         return self.for_widget.retrieve_javascript()
 
+
 class CoreWD(WidgetDescription):
     register = False
 
@@ -797,6 +878,7 @@ class CoreWD(WidgetDescription):
         return "turbogears.widgets.%s" % (cls.__name__)
     full_class_name = property(_get_full_class_name)
 
+
 class RenderOnlyWD(WidgetDescription):
     register = False
     template = """
@@ -812,21 +894,26 @@ class RenderOnlyWD(WidgetDescription):
     def retrieve_css(self):
         return setlike()
 
+
 #############################################################################
 # CSS and JS WidgetDescription's                                            #
 #############################################################################
+
 class CSSLinkDesc(CoreWD, RenderOnlyWD):
     name = "CSS Link"
     for_widget = CSSLink("turbogears", "css/yourstyle.css")
 
+
 class JSLinkDesc(CoreWD, RenderOnlyWD):
     name = "JS Link"
     for_widget = JSLink("turbogears", "js/yourscript.js")
 
+
 class CSSSourceDesc(CoreWD, RenderOnlyWD):
     name = "CSS Source"
     for_widget = CSSSource("""body { font-size:12px; }""")
 
+
 class JSSourceDesc(CoreWD, RenderOnlyWD):
     name = "JS Source"
     for_widget = JSSource("document.title = 'Hello World';")
