@@ -4,11 +4,6 @@ from turbogears.util import setlike, Bunch, request_available
 from turbogears.widgets.base import Widget, CompoundWidget, WidgetsList, \
                                     CoreWD, RenderOnlyWD
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
-
 __all__ = ["InputWidget", "CompoundInputWidget", "RepeatingInputWidget",
            "FormField", "FormFieldsContainer", "CompoundFormField",
            "RepeatingFormField", "Label", "TextField", "PasswordField",
@@ -19,7 +14,6 @@ __all__ = ["InputWidget", "CompoundInputWidget", "RepeatingInputWidget",
            "RepeatingFieldSet", "Form", "TableForm", "ListForm",
            "WidgetsList"]
 
-
 #############################################################################
 # Functions and classes to manage input widgets                             #
 #############################################################################
@@ -131,7 +125,6 @@ def build_name_from_path(path, repeating_marker='-', nesting_marker='.'):
             name.append(item.widget.name)
     return nesting_marker.join(name)
 
-
 ###############################################################################
 # Base class for a widget that can generate input for the app.
 ###############################################################################
@@ -144,18 +137,17 @@ class InputWidget(Widget):
     convert = True
 
     def __init__(self, name=None, validator=None, **params):
-        """Initialize an input widget.
+        """Initializes an input widget.
 
         It accepts the following parameters (besides those listed at params):
 
-        name:
-            Name of the input element. Will be the name of the variable
-            the widget's input will be available at when submitted.
-
-        validator:
-            Formencode validator that will validate and coerce the input
-            this widget generates.
+          name:
+             Name of the input element. Will be the name of the variable
+             the widget's input will be available at when submitted.
 
+          validator:
+             Formencode validator that will validate and coerce the input
+             this widget generates.
         """
         if name is not None and ('-' in name or '.' in name):
             raise ValueError, ("The name of an InputWidget must not contain "
@@ -166,25 +158,25 @@ class InputWidget(Widget):
         if validator:
             self.validator = validator
 
-    def _get_path(self):
+    @property
+    @update_path
+    def path(self):
         return get_path(self, None)[:]
-    _get_path = update_path(_get_path)
-    path = property(_get_path)
 
-    def _get_name_path(self):
+    @property
+    def name_path(self):
         if self.path and getattr(self.path[0].widget, "form", False):
             return self.path[1:]
         else:
             return self.path
-    name_path = property(_get_name_path)
 
-    def _get_is_validated(self):
+    @property
+    def is_validated(self):
         if self.path:
             validated_form = getattr(request, "validated_form", None)
             return self.path[0].widget is validated_form
         else:
             return False
-    is_validated = property(_get_is_validated)
 
     def _retrieve_validator_from_validation_schema(self):
         root_widget = self.path[0].widget
@@ -236,20 +228,20 @@ class InputWidget(Widget):
                 value = self.validator.from_python(value)
         return value
 
+    @update_path
     def display(self, value=None, **params):
         return super(InputWidget, self).display(value, **params)
-    display = update_path(display)
 
-    def _get_fq_name(self):
+    @property
+    @update_path
+    def fq_name(self):
         return build_name_from_path(self.name_path)
-    _get_fq_name = update_path(_get_fq_name)
-    fq_name = property(_get_fq_name)
 
-    def _get_error(self):
+    @property
+    @update_path
+    def error(self):
         errors = getattr(request, "validation_errors", {})
         return retrieve_value_by_path(errors, self.name_path)
-    _get_error = update_path(_get_error)
-    error = property(_get_error)
 
     def update_params(self, d):
         super(InputWidget, self).update_params(d)
@@ -257,7 +249,6 @@ class InputWidget(Widget):
         errors = getattr(request, "validation_errors", {})
         d["error"] = retrieve_value_by_path(errors, self.name_path)
 
-
 class CompoundInputWidget(CompoundWidget, InputWidget):
 
     def update_params(self, params):
@@ -265,32 +256,23 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
         params['error_for'] = lambda f: self.error_for(f, True)
 
     def value_for(self, item, value):
-        """Return the value for a child widget.
-
-        ``item`` is the child widget instance or it's name, ``value`` is a
-        dict containing the value for this compound widget.
-
-        """
+        """Return the value for a child widget. ``item`` is the child
+        widget instance or it's name, ``value`` is a dict containing the
+        value for this compound widget."""
         path = path_from_item(item)
         return retrieve_value_by_path(value, path)
 
     def params_for(self, item, **params):
-        """Return the parameters for a child widget.
-
-        ``item`` is the child widget instance or it's name, ``params`` is a
-        dict containing the parameters passed to this compound widget.
-
-        """
+        """Return the parameters for a child widget. ``item`` is the child
+        widget instance or it's name, ``params`` is a dict containing the
+        parameters passed to this compound widget."""
         path = path_from_item(item)
         return retrieve_params_by_path(params, path)
 
     def error_for(self, item, suppress_errors=False):
-        """Return the Invalid exception associated with a child widget.
-
-        The exception is stored in the request local storage. ``item`` is the
-        child widget instance or it's name.
-
-        """
+        """Return's the Invalid exception associated to a child widget from
+        request local storage. ``item`` is the child widget instance or it's
+        name."""
         if self.is_validated:
             path = path_from_item(item, self.name_path)
             errors = getattr(request, "validation_errors", {})
@@ -304,14 +286,10 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
             return None
 
     def dictify_value(self, value):
-        """Convert value into a dict suitable for propagating values to child
-        widgets.
-
-        If value is a dict it will pass through, if it's another kind of
-        object, attributes which match child widgets' names will tried to be
-        fetched.
-
-        """
+        """Converts a value into a dict suitable for propagating values to
+        child widgets. If value is a dict it will pass thorugh, if it other
+        kind of object, attributes which match child widgets' names will be
+        tried to fetch."""
         if isinstance(value, dict):
             return value
         else:
@@ -329,7 +307,6 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
             value = self.dictify_value(value)
         return super(CompoundWidget, self).adjust_value(value, **params)
 
-
 class RepeatingRange(object):
 
     def __init__(self, repetitions, bunch):
@@ -350,7 +327,6 @@ class RepeatingRange(object):
             self.__bunch.repetition = value
             return value
 
-
 class RepeatingInputWidget(CompoundInputWidget):
     """Base class for a compound widget which can be repeated."""
 
@@ -375,12 +351,9 @@ class RepeatingInputWidget(CompoundInputWidget):
         d["repetitions"] = RepeatingRange(repetitions, path_reference)
 
     def value_for(self, item, value):
-        """Return the value for a child widget.
-
-        ``item`` is the child widget instance or it's name, ``value`` is a dict
-        containing the value for this compound widget.
-
-        """
+        """Return the value for a child widget. ``item`` is the child
+        widget instance or it's name, ``value`` is a dict containing the
+        value for this compound widget."""
         if isinstance(value, list):
             try:
                 value = value[self.path[-1].repetition]
@@ -392,12 +365,9 @@ class RepeatingInputWidget(CompoundInputWidget):
         return retrieve_value_by_path(value, path)
 
     def params_for(self, item, **params):
-        """Return the parameters for a child widget.
-
-        ``item`` is the child widget instance or it's name, ``params`` is a
-        dict containing the parameters passed to this compound widget.
-
-        """
+        """Return the parameters for a child widget. ``item`` is the child
+        widget instance or it's name, ``params`` is a dict containing the
+        parameters passed to this compound widget."""
         item_params = {}
         for k,v in params.iteritems():
             if isinstance(v, list) and k is not "repetitions":
@@ -409,18 +379,14 @@ class RepeatingInputWidget(CompoundInputWidget):
         return retrieve_params_by_path(item_params, path)
 
     def dictify_value(self, value):
-        """Convert list of values into a list of dicts suitable for propagating
-        values to child widgets.
-
-        If value is a list of dicts it will pass through, if it's another kind
-        of object, attributes which match child widgets' names will tried to be
-        fetched.
-
-        """
+        """Converts a list of values into a list of dicts suitable for
+        propagating values to child widgets. If value is a list of dicts it
+        will pass thorugh, if it other kind of object, attributes which match
+        child widgets' names will be
+        tried to fetch."""
         return [super(RepeatingInputWidget, self).dictify_value(v)
                 for v in value]
 
-
 #############################################################################
 # Base classes                                                              #
 #############################################################################
@@ -438,6 +404,7 @@ class FormField(InputWidget):
        widget.
 
     """
+
     _name = "widget"
     label = None
     help_text = None
@@ -456,7 +423,8 @@ class FormField(InputWidget):
             self.label = name.capitalize()
     name = property(_get_name, _set_name)
 
-    def _get_is_required(self):
+    @property
+    def is_required(self):
         validator = self._retrieve_validator_from_validation_schema()
         if validator is None:
             return False
@@ -467,11 +435,10 @@ class FormField(InputWidget):
                 return True
             else:
                 return False
-    is_required = property(_get_is_required)
 
-    def _get_field_id(self):
+    @property
+    def field_id(self):
         return build_name_from_path(self.path, '_', '_')
-    field_id = property(_get_field_id)
 
     def __init__(self, name=None, label=None, help_text=None, **kw):
         super(FormField, self).__init__(name, **kw)
@@ -488,17 +455,19 @@ class FormField(InputWidget):
         super(FormField, self).update_params(d)
         if self.is_required:
             d["field_class"] = " ".join([d["field_class"], "requiredfield"])
+        if d.get('error', None):
+            d["field_class"] = " ".join([d["field_class"], "erroneousfield"])
         if d["css_classes"]:
             d["field_class"] = " ".join([d["field_class"]] + d["css_classes"])
         d["label"] = self.label
         d["help_text"] = self.help_text
         d["field_id"] = self.field_id
 
-
 # A decorator that provides field_for functionality to the
 # decorated FormFieldsContainer's method
 def retrieve_field_for(func):
-    def _retrieve_field_for(self=None, item=None, *args, **kw):
+    @update_path
+    def retrieve_field_for(self=None, item=None, *args, **kw):
         path = path_from_item(item)
         field = self
         for name, index in path:
@@ -512,9 +481,7 @@ def retrieve_field_for(func):
         for i in range(len(path)):
             pop_from_path()
         return returnval
-    _retrieve_field_for = update_path(_retrieve_field_for)
-    return _retrieve_field_for
-
+    return retrieve_field_for
 
 class FormFieldsContainer(CompoundInputWidget):
     """A container for FormFields.
@@ -525,33 +492,34 @@ class FormFieldsContainer(CompoundInputWidget):
 
     It provides the template with 3 useful functions:
        - field_for(name)
-           Returns the child named ``name``.
+           Returns the child named ``name``
        - value_for(name)
            Returns the value for the child named ``name`` (name can also be
-           a widget instance so fields can be iterated).
+           a widget instance so fields can be iterated)
        - params_for(name)
-           Returns the display-time parameters for the child named ``name``
-           (can also be a widget instance so fields can be iterated).
+           Returns the display-time parameters for the child named ``name`` (can
+           also be a widget instance so fields can be iterated)
        - display_field_for(name)
-           Displays the child named ``name`` automatically propagating value
+           displays the child named ``name`` automatically propagating value
            an params. ``name`` can also be a widget instance.
        - error_for(name)
            Returns the validation error the child named ``name`` generated.
            Again, ``name`` can also be a widget instance.
 
     """
+
     member_widgets = ["fields", "hidden_fields"]
     fields = []
     hidden_fields = []
     params = ["disabled_fields"]
     disabled_fields = set()
 
-    def _get_file_upload(self):
+    @property
+    def file_upload(self):
         for widget in self.iter_member_widgets():
             if getattr(widget, "file_upload", False):
                 return True
         return False
-    file_upload = property(_get_file_upload)
 
     def get_field_by_name(self, name, default=None):
         for widget in self.iter_member_widgets():
@@ -559,22 +527,18 @@ class FormFieldsContainer(CompoundInputWidget):
                 return widget
         return default
 
+    @retrieve_field_for
     def display_field_for(self, item, value=None, **params):
         return item.display(value, **params)
-    display_field_for = retrieve_field_for(display_field_for)
 
+    @retrieve_field_for
     def render_field_for(self, item, value=None, format="html", **params):
         return item.render(value, format, **params)
-    render_field_for = retrieve_field_for(render_field_for)
 
     def _field_for(self, item):
-        """Return the member widget named item.
-
-        This function should *only* be used inside a FormFieldsContainer
+        """This function should *only* be used inside a FormFieldsContainer
         template, really, else the path acrobatics will lead to unexpected
-        results.
-
-        """
+        results."""
         field = self.get_field_by_name(item, None)
         if field is None:
             raise ValueError("Field for '%s' not found." % item)
@@ -583,9 +547,11 @@ class FormFieldsContainer(CompoundInputWidget):
     def update_params(self, d):
         super(FormFieldsContainer, self).update_params(d)
         d["display_field_for"] = lambda f: self.display_field_for(f,
-            d["value_for"](f), **d["params_for"](f))
+                                                                  d["value_for"](f),
+                                                                  **d["params_for"](f))
         d["render_field_for"] = lambda f: self.display_field_for(f,
-            d["value_for"](f), **d["params_for"](f))
+                                                                 d["value_for"](f),
+                                                                 **d["params_for"](f))
         d["field_for"] = self._field_for
         visible_fields = []
         hidden_fields = []
@@ -601,15 +567,12 @@ class FormFieldsContainer(CompoundInputWidget):
         d["fields"] = visible_fields
         d["hidden_fields"] = hidden_fields
 
-
 class CompoundFormField(FormFieldsContainer, FormField):
     is_required = False
 
-
 class RepeatingFormField(RepeatingInputWidget, CompoundFormField):
     pass
 
-
 #############################################################################
 # Fields                                                                    #
 #############################################################################
@@ -630,11 +593,9 @@ class Label(FormField):
                             'the label tag'}
     attrs = {}
 
-
 class LabelDesc(CoreWD):
     for_widget = Label(default="Sample Label")
 
-
 class TextField(FormField):
     """A standard, single-line text field."""
 
@@ -653,14 +614,12 @@ class TextField(FormField):
                             ' the input tag'}
     attrs = {}
 
-
 class TextFieldDesc(CoreWD):
     name = "Text Field"
     for_widget = TextField("your_name",
                            default="Your Name Here",
                            attrs=dict(size="30"))
 
-
 class PasswordField(FormField):
     """A password field which masks letters with * characters."""
 
@@ -679,13 +638,11 @@ class PasswordField(FormField):
                             'the password input tag'}
     attrs = {}
 
-
 class PasswordFieldDesc(CoreWD):
     name = "Password Field"
     for_widget = PasswordField("your_secret",
                                default="Top Secret Password")
 
-
 class HiddenField(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -703,12 +660,10 @@ class HiddenField(FormField):
     attrs = {}
     hidden = True
 
-
 class HiddenFieldDesc(CoreWD, RenderOnlyWD):
     name = "Hidden Field"
     for_widget = HiddenField("hidden_one")
 
-
 class FileField(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -728,12 +683,10 @@ class FileField(FormField):
     def display(self, value=None, **params):
         return super(FileField, self).display(None, **params)
 
-
 class FileFieldDesc(CoreWD):
     name = "File Field"
     for_widget = FileField("your_filefield", attrs=dict(size="30"))
 
-
 class Button(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -754,12 +707,10 @@ class Button(FormField):
             d['attrs']['name'] = d["name"]
             d['attrs']['id'] = d["field_id"]
 
-
 class ButtonDesc(CoreWD):
     name = "Button"
     for_widget = Button(default="Button Value")
 
-
 class SubmitButton(Button):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -770,12 +721,10 @@ class SubmitButton(Button):
     />
     """
 
-
 class SubmitButtonDesc(CoreWD):
     name = "Submit Button"
     for_widget = SubmitButton(default="Submit Button Value")
 
-
 class ResetButton(Button):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -786,12 +735,10 @@ class ResetButton(Button):
     />
     """
 
-
 class ResetButtonDesc(CoreWD):
     name = "Reset Button"
     for_widget = ResetButton(default="Reset Button Value")
 
-
 class ImageButton(Button):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -815,12 +762,10 @@ class ImageButton(Button):
     height = None
     alt = None
 
-
 class ImageButtonDesc(CoreWD):
     name = "Image Button"
     for_widget = ImageButton("your_image_button", src="/tg_static/images/toolbox_logo.png")
 
-
 class CheckBox(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -852,7 +797,6 @@ class CheckBox(FormField):
         if value:
             d['attrs']['checked'] = 'checked'
 
-
 class CheckBoxDesc(CoreWD):
     for_widget = CheckBox(name="your_checkbox",
                           default=True,
@@ -866,7 +810,6 @@ class CheckBoxDesc(CoreWD):
     </div>
     """
 
-
 class TextArea(FormField):
     template = """
     <textarea xmlns:py="http://purl.org/kid/ns#"
@@ -888,14 +831,12 @@ class TextArea(FormField):
     cols = 50
     attrs = {}
 
-
 class TextAreaDesc(CoreWD):
     name = "Text Area"
     for_widget = TextArea(name="your_textarea",
                           default="Your Comment Here",
                           rows=5, cols=40)
 
-
 class SelectionField(FormField):
     # an empty selection doesn't submit anything
     no_input_if_empty = True
@@ -999,7 +940,6 @@ class SelectionField(FormField):
                     return True
         return False
 
-
 class SingleSelectField(SelectionField):
     template = """
     <select xmlns:py="http://purl.org/kid/ns#"
@@ -1026,7 +966,6 @@ class SingleSelectField(SelectionField):
                             ' the select tag'}
     attrs = {}
 
-
 class SingleSelectFieldDesc(CoreWD):
     name = "Single Select Field"
     for_widget = SingleSelectField("your_single_select_field",
@@ -1036,7 +975,6 @@ class SingleSelectFieldDesc(CoreWD):
                                             (4, "Ruby")],
                                    default=2)
 
-
 class MultipleSelectField(SelectionField):
     template = """
     <select xmlns:py="http://purl.org/kid/ns#"
@@ -1066,7 +1004,6 @@ class MultipleSelectField(SelectionField):
     attrs = {}
     size = 5
 
-
 class MultipleSelectFieldDesc(CoreWD):
     name = "Multiple Select Field"
     for_widget = MultipleSelectField("your_multiple_select_field",
@@ -1076,7 +1013,6 @@ class MultipleSelectFieldDesc(CoreWD):
                                               ("d", "Ruby")],
                                      default=["a","c","d"])
 
-
 class RadioButtonList(SelectionField):
     template = """
     <ul xmlns:py="http://purl.org/kid/ns#"
@@ -1100,13 +1036,11 @@ class RadioButtonList(SelectionField):
     params_doc = {'list_attrs' : 'Extra (X)HTML attributes for the ul tag'}
     list_attrs = {}
 
-
 class RadioButtonListDesc(CoreWD):
     name = "RadioButton List"
     for_widget = RadioButtonList("your_radiobutton_list",
         options=list(enumerate("Python Java Pascal Ruby".split())), default=3)
 
-
 class CheckBoxList(SelectionField):
     template = """
     <ul xmlns:py="http://purl.org/kid/ns#"
@@ -1131,14 +1065,12 @@ class CheckBoxList(SelectionField):
     params_doc = {'list_attrs' : 'Extra (X)HTML attributes for the ul tag'}
     list_attrs = {}
 
-
 class CheckBoxListDesc(CoreWD):
     name = "CheckBox List"
     for_widget = CheckBoxList("your_checkbox_list",
         options=list(enumerate("Python Java Pascal Ruby".split())),
         default=[0,3])
 
-
 class FieldSet(CompoundFormField):
     template="""
     <fieldset xmlns:py="http://purl.org/kid/ns#"
@@ -1151,7 +1083,7 @@ class FieldSet(CompoundFormField):
         />
         <div py:for="field in fields">
             <label class="fieldlabel" for="${field.field_id}" py:content="field.label" />
-            <span py:content="field.display(value_for(field), **params_for(field))" />
+            <span py:replace="field.display(value_for(field), **params_for(field))" />
             <span py:if="error_for(field)" class="fielderror" py:content="error_for(field)" />
             <span py:if="field.help_text" class="fieldhelp" py:content="field.help_text" />
         </div>
@@ -1161,7 +1093,6 @@ class FieldSet(CompoundFormField):
     params_doc = {'legend' : 'Text to display as the legend for the fieldset'}
     legend = None
 
-
 class FieldSetDesc(CoreWD):
     name = "FieldSet"
     for_widget = FieldSet("your_fieldset",
@@ -1171,7 +1102,6 @@ class FieldSetDesc(CoreWD):
                                   TextField(name="upper_limit",
                                             label="Upper Limit")])
 
-
 class RepeatingFieldSet(RepeatingFormField):
     template="""
     <div xmlns:py="http://purl.org/kid/ns#">
@@ -1185,7 +1115,7 @@ class RepeatingFieldSet(RepeatingFormField):
         />
         <div py:for="field in fields">
             <label class="fieldlabel" for="${field.field_id}" py:content="field.label" />
-            <span py:content="field.display(value_for(field), **params_for(field))" />
+            <span py:replace="field.display(value_for(field), **params_for(field))" />
             <span py:if="error_for(field)" class="fielderror" py:content="error_for(field)" />
             <span py:if="field.help_text" class="fieldhelp" py:content="field.help_text" />
         </div>
@@ -1196,7 +1126,6 @@ class RepeatingFieldSet(RepeatingFormField):
     params_doc = {'legend' : 'Text to display as the legend for the fieldset'}
     legend = None
 
-
 class RepeatingFieldSetDesc(CoreWD):
     name = "Repeating FieldSet"
     for_widget = RepeatingFieldSet("your_repeating_fieldset",
@@ -1207,7 +1136,6 @@ class RepeatingFieldSetDesc(CoreWD):
                                            TextField(name="upper_limit",
                                                      label="Upper Limit")])
 
-
 #############################################################################
 # Forms                                                                     #
 #############################################################################
@@ -1227,7 +1155,7 @@ class Form(FormFieldsContainer):
     action = ''
     method = 'post'
     form_attrs = {}
-    use_name = True
+    use_name = False # because this is deprecated in HTML and invalid in XHTML
     submit_text = None
 
     def update_params(self, d):
@@ -1241,10 +1169,9 @@ class Form(FormFieldsContainer):
         if self.validator:
             return self.validator.to_python(value, state)
 
-    def _get_errors(self):
+    @property
+    def errors(self):
         return getattr(request, 'validation_errors', None)
-    errors = property(_get_errors)
-
 
 class TableForm(Form):
     """Form with simple table layout."""
@@ -1286,7 +1213,6 @@ class TableForm(Form):
     params_doc = {'table_attrs' : 'Extra (X)HTML attributes for the Table tag'}
     table_attrs = {}
 
-
 class TableFormDesc(CoreWD):
     name = "Table Form"
     full_class_name = "turbogears.widgets.TableForm"
@@ -1306,11 +1232,10 @@ class TableFormDesc(CoreWD):
     </div>
     """
 
-    [expose()]
+    @expose()
     def save(self, **kw):
         return """Received data from TableForm:<br />%r""" % kw
 
-
 class ListForm(Form):
     """Form with simple list layout."""
 
@@ -1344,7 +1269,6 @@ class ListForm(Form):
     params_doc = {'list_attrs' : 'Extra (X)HTML attributes for the ul tag'}
     list_attrs = {}
 
-
 class ListFormDesc(CoreWD):
     name = "List Form"
     full_class_name = "turbogears.widgets.ListForm"
@@ -1364,6 +1288,6 @@ class ListFormDesc(CoreWD):
     </div>
     """
 
-    [expose()]
+    @expose()
     def save(self, **kw):
         return """Received data from ListForm:<br />%r""" % kw
