@@ -1,50 +1,27 @@
-import copy
 import threading
+import copy
 import warnings
 
-from inspect import isclass
-from itertools import count, ifilter
+# FIXME-dbr: we need to make kid-support pluggable or such.
+try:
+    import kid
+except ImportError:
+    pass
 from new import instancemethod
-
-import kid
-
+from itertools import ifilter, count
 from turbogears import validators
 from turbogears.util import setlike
 from formencode.schema import Schema
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
-
 __all__ = ["MetaWidget", "load_kid_template"]
 
 param_prefix = '_param_'
 
 class MetaWidget(type):
-    """The meta class for widgets."""
-
     def __new__(cls, name, bases, dct):
-        # update_data has been deprecated
-        if 'update_data' in dct and name != "Widget":
-            warnings.warn(
-                "update_data has been renamed update_params, please "
-                "rename your method. "
-                "Note: this warning will be removed once 1.0 is "
-                "released and your actual code will stop working.",
-                DeprecationWarning, 2)
         # Makes sure we get the union of params and member_widgets
         # from all our bases.
         params = setlike(dct.get('params', []))
-        # template_vars has been deprecated
-        if 'template_vars' in dct:
-            params.add_all(dct['template_vars'])
-            warnings.warn(
-                "Use of template_vars inside a widget is deprecated, "
-                "use params instead. "
-                "Note: this warning will be removed once 1.0 is "
-                "released and your actual code will stop working.",
-                DeprecationWarning, 2)
         member_widgets = setlike(dct.get('member_widgets', []))
         compound = False
         for base in bases:
@@ -62,8 +39,6 @@ class MetaWidget(type):
                 pass
         params = list(params)
         dct['params'] = params
-        #XXX: Remove when deprecation is effective
-        dct['template_vars'] = params
         if compound:
             dct['member_widgets'] = list(member_widgets)
         # Pick params_doc from all bases giving priority to the widget's own
@@ -108,7 +83,7 @@ class ParamDescriptor(object):
         setattr(obj, self.param_name, value)
 
 def lockwidget(self, *args, **kw):
-    """Set this widget as locked the first time it's displayed."""
+    "Sets this widget as locked the first time it's displayed."
     gotlock = self._displaylock.acquire(False)
     if gotlock:
         del self.display
@@ -119,12 +94,11 @@ def lockwidget(self, *args, **kw):
     return output
 
 def _decorate_widget_init(func):
-    """Decorator for a widgets __init__ method.
-
+    """
     Ensures that the display method for the instance is overridden by
-    lockwidget, that an eventual validator dict is applied to the widget
-    validator and that a validation schema is generated for compound widgets.
-
+    lockwidget, that an eventual validator dict is applied to the
+    widget validator and that a validation schema is generated for compound
+    widgets.
     """
     def widget_init(self, *args, **kw):
         self.display = instancemethod(lockwidget, self, self.__class__)
@@ -192,13 +166,21 @@ widget_instance_serial = count()
 
 # XXX: Maybe Widget should have a __del__ method to unload any template that
 #      has been loaded during the instances initialization?
+# FIXME-dbr: This is returning nonsens in case KID isn't imported because
+# we want to get rid of KID as a dependency - and as long as the widgets
+# are in TG itself, that leads no not-so-beautiful compromises.
 def load_kid_template(t, modname=None):
-    """Load the given template into the given module name.
-
-    If modname is None, an unique one will be generated.
-    Returns a tuple (compiled_tmpl, template_text (or modulepath).
-
     """
+    Loads the given template into the given module name, if modname is None,
+    an unique one will be generated.
+    Returns a tuple (compiled_tmpl, template_text (or modulepath)
+    """
+    # this is a dummy-value-return-thingy, see above comment
+    try:
+        kid
+    except NameError:
+        return None, None
+    
     if isinstance(t, basestring) and "<" in t:
         if not modname:
             modname = 'instance_template_%d' % widget_instance_serial.next()
@@ -210,16 +192,14 @@ def load_kid_template(t, modname=None):
 # Schema generation support                                                 #
 #############################################################################
 class NullValidator(validators.FancyValidator):
-    """A do-nothing validator.
-
-    Used as a placeholder for fields with no validator so they don't get
-    stripped by the Schema.
-
+    """
+    A do-nothing validator. Used as a placeholder for fields with
+    no validator so they don't get stripped by the Schema.
     """
     if_missing = None
 
 def copy_schema(schema):
-    """Recursively copy a schema."""
+    """ recursively copies a schema """
     new_schema = copy.copy(schema)
     new_schema.pre_validators = schema.pre_validators[:]
     new_schema.chained_validators = schema.chained_validators[:]
@@ -232,18 +212,11 @@ def copy_schema(schema):
     return new_schema
 
 def merge_schemas(to_schema, from_schema, inplace=False):
-    """Recursively merge from_schema into to_schema
-
-    Takes care of leaving to_schema intact if inplace is False (default).
-    Returns a new Schema instance if inplace is False or to_schema is a Schema
-    class not an instance or the changed to_schema.
-
     """
-    # Nested schemas may be Schema (sub-)classes instead of instances.
-    # Copying Schema classes does not work, so we just create an instance.
-    if isclass(to_schema) and issubclass(to_schema, validators.Schema):
-        to_schema = to_schema()
-    elif not inplace:
+    Recursively merges from_schema into to_schema taking care of leaving
+    to_schema intact if inplace is False (default).
+    """
+    if not inplace:
         to_schema = copy_schema(to_schema)
 
     # Recursively merge child schemas
@@ -264,15 +237,15 @@ def merge_schemas(to_schema, from_schema, inplace=False):
     return to_schema
 
 def add_field_to_schema(schema, widget):
-    """Add widget's validator if any to the given schema."""
+    """ Adds a widget's validator if any to the given schema """
     name = widget.name
     if widget.validator is not None:
         if isinstance(widget.validator, validators.Schema):
-            # Schema instance or class, might need to merge 'em...
+            # Schema instance, might need to merge 'em...
             if widget.name in schema.fields:
-                assert (isinstance(schema.fields[name], validators.Schema) or
-                    issubclass(schema.fields[name], validators.Schema)
-                    ), "Validator for '%s' should be a Schema subclass" % name
+                assert isinstance(
+                    schema.fields[name], validators.Schema
+                ), "Validator for '%s' should be a Schema subclass" % name
                 v = merge_schemas(schema.fields[name], widget.validator)
             else:
                 v = widget.validator
@@ -284,18 +257,12 @@ def add_field_to_schema(schema, widget):
         schema.add_field(name, NullValidator())
 
 def generate_schema(schema, widgets):
-    """Generate or extend a copy of schema with all validators of all widgets.
-
-    schema may be a schema instance or class, widgets is a list of widgets
-    instances.
-    Returns the new schema instance.
-
+    """
+    Generates or extends a copy of schema with all the validators from
+    the widget in the widgets list.
     """
     if schema is None:
         schema = validators.Schema()
-    elif isclass(schema) and issubclass(schema, validators.Schema):
-        # allows to attach Schema (sub-)classes to widgets.
-        schema = schema()
     else:
         schema = copy_schema(schema)
     for widget in widgets:
@@ -304,11 +271,9 @@ def generate_schema(schema, widgets):
     return schema
 
 def can_add_field(schema, field_name):
-    """Checks if we can safely add a field.
-
-    Makes sure we're not overriding any field in the Schema. NullValidators
-    are ok to override.
-
+    """
+    Checks if we can safely add a field. Makes sure we're not overriding
+    any field in the Schema. NullValidators are ok to override.
     """
     current_field = schema.fields.get(field_name)
     return bool(current_field is None or
