@@ -1,35 +1,21 @@
 import re
-import cherrypy
-
-import turbogears.validators as validators
+from turbogears.testutil import catch_validation_errors, start_server
+import turbogears
+from turbogears import testutil
 import turbogears.widgets as widgets
-
-from turbogears.testutil import catch_validation_errors
+import turbogears.validators as validators
 from turbogears.widgets.meta import copy_schema
 
-
-oldrequest = None
-int_validator = validators.Int(if_empty=None)
-s_validator = validators.Schema(age=int_validator, ignore_key_missing=True)
-
-
-class Request:
-    validation_errors = {}
-
 def setup_module():
-    global oldrequest
-    oldrequest = cherrypy.request
-    cherrypy.request = Request()
-
-def teardown_module():
-    global oldrequest
-    cherrypy.request = oldrequest
+    start_server()
 
 
 #XXX: We ignore missing keys to make passing value easier in tests
 class TestSchema(validators.Schema):
     ignore_key_missing = True
 
+int_validator = validators.Int(if_empty=None)
+s_validator = validators.Schema(age=int_validator, ignore_key_missing=True)
 
 class TestNestedWidgets:
 
@@ -47,30 +33,47 @@ class TestNestedWidgets:
         ], validator = TestSchema()),
     ], validator = TestSchema())
 
+
+    class MyRoot(turbogears.controllers.RootController):
+
+        @turbogears.expose()
+        def index(self, test_id):
+            if test_id == 'age':
+               data = dict(age=22)
+            elif test_id == 'sub2':
+               data = dict(sub=dict(sub2=dict(age=22)))
+            elif test_id == 'sub':
+               data = dict(dict(sub=dict(age=22)))
+            return self.form.render(data)
+
+    MyRoot.form = form
+
+
     def test_display(self):
         """
         Checks if names fo the widgets are set correctly depending on their
         path.
         """
-        output = self.form.render(dict(sub=dict(sub2=dict(age=22))), format='xhtml')
+        app = testutil.make_app(self.MyRoot)
+        response = app.get('/?test_id=sub2')
+        output = response.body
         value_p = 'value="22"'
         name_p = 'name="sub.sub2.age"'
         assert (re.compile('.*'.join([value_p, name_p])).search(output) or
                 re.compile('.*'.join([name_p, value_p])).search(output))
 
-        output = self.form.render(dict(sub=dict(age=22)), format='xhtml')
+        response = app.get('/?test_id=sub')
+        output = response.body
         value_p = 'value="22"'
         name_p = 'name="sub.age"'
+        id_p = 'id="myform_sub_age"'
         assert (re.compile('.*'.join([value_p, name_p])).search(output) or
                 re.compile('.*'.join([name_p, value_p])).search(output))
-
-        output = self.form.render(dict(sub=dict(age=22)), format='xhtml')
-        id_p = 'id="myform_sub_age"'
-        name_p = 'name="sub.age"'
         assert (re.compile('.*'.join([value_p, id_p])).search(output) or
                 re.compile('.*'.join([id_p, value_p])).search(output))
 
-        output = self.form.render(dict(age=22), format='xhtml')
+        response = app.get('/?test_id=age')
+        output = response.body
         value_p = 'value="22"'
         name_p = 'name="age"'
         assert (re.compile('.*'.join([value_p, name_p])).search(output) or
@@ -85,6 +88,7 @@ class TestNestedWidgets:
         assert errors.pop('age', False)
         assert not errors
 
+
     def test_validate_sub(self):
         values = dict(sub=dict(age="twenty"))
 
@@ -99,6 +103,7 @@ class TestNestedWidgets:
         assert errors.pop('age', False)
         assert not errors
 
+
     def test_validate_sub2(self):
         values = dict(sub=dict(sub2=dict(age="twenty")))
 
@@ -116,6 +121,7 @@ class TestNestedWidgets:
         assert errors.pop('age', False)
         assert not errors
 
+
     def test_validate_sub_and_sub2(self):
         values = dict(sub=dict(age="fhg", sub2=dict(age="twenty")))
 
@@ -133,6 +139,7 @@ class TestNestedWidgets:
         assert errors.pop('age', False)
         assert not errors
 
+
     def test_good_values(self):
         values = dict(age=22, sub=dict(sub2=dict(age=20)))
 
@@ -143,6 +150,7 @@ class TestNestedWidgets:
         assert errors == {}
         assert values['age'] == 22
 
+
     def test_good_and_bad_values(self):
         values = dict(age="ddd", sub=dict(age="20", sub2=dict()))
 
@@ -154,6 +162,7 @@ class TestNestedWidgets:
         #assert values['sub']['age'] == 20
 
 
+
 class TestNestedWidgetsWSchemaValidation:
 
     form = widgets.TableForm(
@@ -181,6 +190,7 @@ class TestNestedWidgetsWSchemaValidation:
         ]
     )
 
+
     def test_validate_sub_schema(self):
         values = dict(sub=dict(age="twenty"))
 
@@ -257,7 +267,6 @@ class TestNestedWidgetsWSchemaValidation:
         assert not errors
         assert not errors
 
-
 class TestNestedWidgetsWMixedValidation:
 
     form = widgets.TableForm(
@@ -325,23 +334,24 @@ class TestNestedWidgetsWMixedValidation:
         assert not errors
 
 
+
+
+
+
 class InnerSchema(validators.Schema):
     ignore_key_missing = True
     age = int_validator
 
-
 class MiddleSchema(validators.Schema):
     ignore_key_missing = True
     age = int_validator
     sub2 = InnerSchema()
 
-
 class OuterSchema(validators.Schema):
     ignore_key_missing = True
     age = int_validator
     sub = MiddleSchema()
 
-
 class TestNestedSchemaValidators:
 
     #XXX: Age is always validated by the nested schemas, number is
@@ -435,24 +445,7 @@ class TestNestedSchemaValidators:
         values = values['sub2']
         assert (values["age"], values['number']) == (33, 34)
 
-
 def test_copy_schema():
-    """Test that a validator schema can be copied."""
     class UserSchema(validators.Schema):
         user_name = validators.PlainText()
     schema = copy_schema(UserSchema())
-
-def test_copy_nested_schema():
-    """Test that a nested validator schema can be copied."""
-    class PersonSchema(validators.Schema):
-        class namefields(validators.Schema):
-            firstname = validators.PlainText()
-            lastname = validators.PlainText()
-        class parents(validators.Schema):
-            class father(validators.Schema):
-                firstname = validators.PlainText()
-                lastname = validators.PlainText()
-            class mother(validators.Schema):
-                firstname = validators.PlainText()
-                lastname = validators.PlainText()
-    schema = copy_schema(PersonSchema())
