@@ -1,26 +1,28 @@
 import itertools
-
 import cherrypy
-
-from turbogears import widgets, validators
+from turbogears import widgets, validators, view, testutil, controllers, expose
 from turbogears.testutil import catch_validation_errors
 
-import sets
-
-class Request:
-    input_values = {}
-    validation_errors = {}
-
-oldrequest = None
-
 def setup_module():
-    global oldrequest
-    oldrequest = cherrypy.request
-    cherrypy.request = Request()
+    testutil.start_server()
 
 def teardown_module():
-    global oldrequest
-    cherrypy.request = oldrequest
+    testutil.stop_server()
+
+
+def test_rendering_without_engine():
+    """Helpful error when rendering widgets with no templating engine loaded"""
+    from turbogears import view
+    engines = view.engines
+    view.engines = {}
+    try:
+        widgets.CSSLink("foo")()
+    except Exception, msg:
+        msg = str(msg)
+    else:
+        msg = 'No error'
+    view.engines = engines
+    assert 'templating engine is not yet loaded' in msg
 
 def test_label():
     """Tests simple labels"""
@@ -37,23 +39,47 @@ def test_default_value():
     output = textfield.render(format='xhtml')
     assert 'value="ed kowalczyk"' in output
 
+def test_callable_default_value():
+    """Widgets can have a callable default value"""
+    textfield = widgets.TextField("name", default=lambda: "chad taylor")
+    output = textfield.render(format='xhtml')
+    assert 'value="chad taylor"' in output
+
+def test_false_default_value():
+    """Widgets can have a default value that evaluates to False"""
+    textfield = widgets.TextField("name")
+    assert textfield.default is None
+    output = textfield.render(format='xhtml')
+    assert 'value' not in output
+    textfield = widgets.TextField("name", default=0)
+    assert textfield.default == 0
+    output = textfield.render(format='xhtml')
+    assert 'value="0"' in output
+    textfield = widgets.TextField("name", default="")
+    assert textfield.default == ""
+    output = textfield.render(format='xhtml')
+    assert 'value=""' in output
+    msfield = widgets.MultipleSelectField("name", default=[], options=(1,2,3))
+    assert msfield.default == []
+    output = msfield.render(format='xhtml')
+    assert 'selected' not in output
+
 def test_labeltext():
-    "Label text defaults to the capitalized name"
+    """Label text defaults to the capitalized name"""
     textfield = widgets.TextField("name")
     assert textfield.label == "Name"
 
 def test_validation():
-    "Values can be converted to/from python values"
+    """Values can be converted to/from Python values"""
     textfield = widgets.TextField("age", validator=validators.Int())
     output = textfield.render(2, format="xhtml")
     assert 'value="2"' in output
     value = "2"
     value = textfield.validator.to_python(value)
-    print value
     assert value == 2
 
 def test_unicode_input():
-    "Unicode values are rendered correctly"
+    """Unicode values are rendered correctly"""
     tf = widgets.TextField("name", validator=validators.UnicodeString())
     output = tf.render(u'Pete \u011C', format='xhtml')
     assert 'value="Pete \xc4\x9c"' in output
@@ -63,7 +89,7 @@ def test_unicode_input():
     except ValueError, e:
         pass
     else:
-        assert 0, "ValueError not raised: non-unicode input not detected"
+        assert False, "ValueError not raised: non-unicode input not detected"
     #tf2 = widgets.TextField("name", validator=validators.String())
 
 # simon: failed inputs are no longer being removed.
@@ -79,11 +105,11 @@ def test_unicode_input():
 #    assert not values.has_key("age")
 
 def test_widget_css():
-    "Widgets can require CSS resources"
+    """Widgets can require CSS resources"""
     css = widgets.CSSLink(mod=widgets.static, name="foo.css")
     css2 = widgets.CSSLink(mod=widgets.static, name="foo.css")
     assert css == css2
-    cssset = sets.Set()
+    cssset = set()
     cssset.add(css)
     cssset.add(css2)
     assert len(cssset) == 1
@@ -104,25 +130,111 @@ def test_widget_css():
     assert 'h1 { color: black }' in rendered
 
 def test_widget_js():
-    "Widgets can require JavaScript resources"
+    """Widgets can require JavaScript resources"""
     js = widgets.JSLink(mod=widgets.static, name="foo.js")
     js2 = widgets.JSLink(mod=widgets.static, name="foo.js")
-    assert js == js2
-    js3 = widgets.CSSLink(mod=widgets.static, name="bar.js")
-    assert js3 != js2
-    js4 = widgets.JSSource(src="foo.js")
-    assert js != js4
-    rendered = js.render(format='xhtml').strip()
+    assert js2 == js2
+    js2 = widgets.JSLink(mod=widgets.static, name="bar.js")
+    assert js2 != js
+    js2 = widgets.CSSLink(mod=widgets.static, name="foo.js")
+    assert js2 != js
+    js2 = widgets.JSSource(src="foo.js")
+    assert js2 != js
+    rendered = js.render(format='xhtml')
     expected = '<script src="/tg_widgets/turbogears.widgets/foo.js"' \
-               ' type="text/javascript"></script>'
+        ' type="text/javascript"></script>'
     assert rendered == expected
-    js = widgets.JSSource("alert('hello');")
-    rendered = js.render(format='xhtml').strip()
-    expected = """<script type="text/javascript">alert('hello');</script>"""
+    js3 = widgets.JSLink(mod=widgets.static, name="foo.js",
+        defer=False, charset=None)
+    assert js3 == js
+    rendered = js3.render(format='xhtml')
     assert rendered == expected
+    js3 = widgets.JSLink(mod=widgets.static, name="foo.js", defer=True)
+    assert js3 != js
+    rendered = js3.render(format='html').lower()
+    assert ' defer' in rendered \
+        and expected == rendered.replace(' defer', '', 1)
+    rendered = js3.render(format='xhtml')
+    assert ' defer="defer"' in rendered \
+        and expected == rendered.replace(' defer="defer"', '', 1)
+    js3 = widgets.JSLink(mod=widgets.static, name="foo.js", charset='Latin-1')
+    assert js3 != js
+    rendered = js3.render(format='xhtml')
+    assert ' charset="Latin-1"' in rendered \
+        and expected == rendered.replace(' charset="Latin-1"', '', 1)
+    js3 = widgets.JSSource("alert('hello');")
+    assert js3 != js and js3 != js2
+    rendered = js3.render(format='xhtml')
+    expected = '<script type="text/javascript">alert(\'hello\');</script>'
+    assert rendered == expected
+    js4 = widgets.JSSource("alert('hello');", defer=False)
+    assert js4 == js3
+    rendered = js4.render(format='xhtml')
+    assert rendered == expected
+    js4 = widgets.JSSource("alert('hello');", defer=True)
+    assert js4 != js3
+    rendered = js4.render(format='html').lower()
+    assert ' defer' in rendered \
+        and expected == rendered.replace(' defer', '', 1)
+    rendered = js4.render(format='xhtml')
+    assert ' defer="defer"' in rendered \
+        and expected == rendered.replace(' defer="defer"', '', 1)
+
+def test_widgetslist_init():
+    """Widget lists can be declared in various ways."""
+    w = widgets.Widget(name="foo")
+    widgetlist = widgets.WidgetsList(w)
+    assert len(widgetlist) == 1
+    assert widgetlist[0] == w
+    widgetlist = widgets.WidgetsList([w])
+    assert len(widgetlist) == 1
+    assert widgetlist[0] == w
+    w2 = widgets.Widget(name="bar")
+    widgetlist = widgets.WidgetsList(w, w2)
+    assert len(widgetlist) == 2
+    assert widgetlist[0] == w
+    assert widgetlist[1] == w2
+    widgetlist = widgets.WidgetsList([w, w2])
+    assert len(widgetlist) == 2
+    assert widgetlist[0] == w
+    assert widgetlist[1] == w2
+    class W(widgets.WidgetsList):
+        foo = w
+        bar = w2
+    widgetlist = W()
+    assert len(widgetlist) == 2
+    assert widgetlist[0] == w
+    assert widgetlist[1] == w2
+    w3, w4 = widgets.Widget(), widgets.Widget()
+    widgetlist = W([w3, w4])
+    assert len(widgetlist) == 4
+    assert widgetlist[0] == w
+    assert widgetlist[1] == w2
+    assert widgetlist[2] == w3
+    assert widgetlist[3] == w4
+
+def test_widgetslist_inheritance():
+    """Widget lists can inherit fields."""
+    w = widgets.Widget(name="foo")
+    class W(widgets.WidgetsList):
+        foo = w
+    w2 = widgets.Widget(name="bar")
+    class W2(W):
+        bar = w2
+    widgetlist = W2()
+    assert len(widgetlist) == 2
+    assert widgetlist[0] == w
+    assert widgetlist[1] == w2
+    w3, w4 = widgets.Widget(), widgets.Widget()
+    widgetlist = W2([w3, w4])
+    assert len(widgetlist) == 4
+    assert widgetlist[0] == w
+    assert widgetlist[1] == w2
+    assert widgetlist[2] == w3
+    assert widgetlist[3] == w4
 
 def test_widget_url():
-    "It might be needed to insert an URL somewhere"
+    """It might be needed to insert an URL somewhere"""
     url = widgets.URLLink(link='http://www.turbogears.org')
     rendered = url.render(format='xhtml')
     expected = """<a href="http://www.turbogears.org"></a>"""
@@ -165,7 +277,7 @@ def test_submit():
     assert 'value="Discard"' in r
 
 def test_threadsafety():
-    "Widget attributes can't be changed after init, for threadsafety"
+    """Widget attributes can't be changed after init, for threadsafety"""
     w = widgets.TextField("bar")
     w.display()
     try:
@@ -175,7 +287,7 @@ def test_threadsafety():
         pass
 
 def test_checkbox():
-    "A CheckBox has not a value and is not checked by default"
+    """A CheckBox has not a value and is not checked by default"""
     w = widgets.CheckBox("foo")
     output = w.render(format='xhtml')
     assert 'name="foo"' in output
@@ -196,16 +308,22 @@ def test_checkbox():
     assert value == 'checked'
 
 def test_field_class():
-    "The class of a field corresponds to the name of its python class"
+    """The class of a field corresponds to the name of its Python class"""
     w = widgets.TextField("bar")
     output = w.render(format='xhtml')
-    assert 'class="%s"' % w.__class__.__name__
+    assert 'class="%s"' % w.__class__.__name__.lower() in output
 
 def test_field_id():
-    "The id of a field corresponds to the name of the field"
+    """The id of a field corresponds to the name of the field"""
     w = widgets.TextField("bar")
     output = w.render(format='xhtml')
-    assert 'id="bar"'
+    assert 'id="bar"' in output
+
+def test_override_name():
+    """The name of a widget can be overridden when displayed"""
+    w = widgets.DataGrid(name="bar", fields=[])
+    output = w.render([], name='foo', format='xhtml')
+    assert 'id="foo"' in output
 
 def test_selection_field():
     """A selection field presents a list of options that can be changed
@@ -285,7 +403,7 @@ class TestParams:
     def test_building_params(self):
         """
         Tests that the list of params is built correctly. Must be the
-        union of all params from all bases
+        union of all params from all bases.
         """
         # for easy equivalenece testing
         self.C.params.sort()
@@ -297,9 +415,8 @@ class TestParams:
         at the subclass declarartion default to None.
         """
         a = self.A()
-        assert a.a == None
+        assert a.a is None
         output = a.render(format='xhtml')
-        print output
         assert 'a=' not in output
 
     def test_overridal(self):
@@ -312,15 +429,13 @@ class TestParams:
         c = self.C(a="test")
         assert c.a == "test"
         output = c.render(format='xhtml')
-        print output
         assert 'a="test"' in output
         output = c.render(a="another", format='xhtml')
-        print output
         assert 'a="another"' in output
 
 def test_template_overridal():
-    """ Tests that we can override an instances template at construction time
-    and get it automatically compiled """
+    """Tests that we can override an instances template at construction time
+    and get it automatically compiled."""
     new_template = """
     <label xmlns:py="http://purl.org/kid/ns#"
         for="${name}"
@@ -352,17 +467,32 @@ def test_simple_widget_attrs():
     assert 'onclick' not in output
 
 def test_textfield():
+    """Test form rendering inside a request."""
+    #form name is prepended to element ids only when running inside a request.
     class MyField(widgets.WidgetsList):
         blonk = widgets.TextField()
+    tf = widgets.ListForm(fields=MyField())
+
     class MyFieldOverrideName(widgets.WidgetsList):
         blonk = widgets.TextField(name="blink")
+    tf2 = widgets.ListForm(fields=MyFieldOverrideName())
 
-    tf = widgets.ListForm(fields=MyField())
-    r = tf.render(format='xhtml')
+    class MyRoot(controllers.RootController):
+        @expose()
+        def fields(self):
+            return tf.render(format='xhtml')
+
+        @expose()
+        def override(self):
+            return tf2.render(format='xhtml')
+
+    testutil.mount(MyRoot(), '/')
+    app = testutil.make_app()
+    r = app.get('/fields')
     assert 'name="blonk"' in r
     assert 'id="form_blonk"' in r
-    tf = widgets.ListForm(fields=MyFieldOverrideName())
-    r = tf.render(format='xhtml')
+
+    r = app.get('/override')
     assert 'name="blink"' in r
     assert 'id="form_blink"' in r
 
@@ -403,7 +533,7 @@ def test_render_field_for():
 def test_css_classes():
     """A FormField supports css_classes, they are added after the original
     class. They can be provided at construction or at display time, the latter
-    overrides the former but attrs overrides everything"""
+    overrides the former but attrs overrides everything."""
     w = widgets.TextField(name="foo")
     output = w.render(format='xhtml')
     assert 'class="textfield"' in output
@@ -423,15 +553,15 @@ def test_css_classes():
     assert 'class="funny"' in output
 
 def test_ticket272():
-    """ TextFields with a "name" attribute = "title" should be OK """
+    """TextFields with a "name" attribute = "title" should be OK."""
     w = widgets.TableForm(fields=[widgets.TextField(name='title')])
     output = w.render(format='xhtml')
     assert 'value' not in output
 
 
 class TestSchemaValidation:
-    """ Tests the validation of a CompoundWidget is done correctly with a
-    Schema validator and no validators on the child widgets. """
+    """Tests the validation of a CompoundWidget is done correctly with a
+    Schema validator and no validators on the child widgets."""
     class Fields(widgets.WidgetsList):
         name = widgets.TextField()
         age = widgets.TextField()
@@ -453,7 +583,6 @@ class TestSchemaValidation:
                       passwd2="fado")
 
         values, errors = catch_validation_errors(self.form, values)
-        print values
         assert values['age'] == 99
         assert not errors
 
@@ -462,13 +591,12 @@ class TestSchemaValidation:
                       passwd2="fadO")
 
         values, errors = catch_validation_errors(self.form, values)
-        print errors
         assert "passwd2" in errors.keys()
 
 class TestSchemaValidationWithChildWidgetsValidators:
-    """ Tests the validation of a CompoundWidget is done correctly with a
+    """Tests the validation of a CompoundWidget is done correctly with a
     Schema validator and independent validators on the each of the child
-    widgets. """
+    widgets."""
     class Fields(widgets.WidgetsList):
         name = widgets.TextField(validator = validators.UnicodeString())
         age = widgets.TextField(validator = validators.Int())
@@ -478,7 +606,6 @@ class TestSchemaValidationWithChildWidgetsValidators:
     class FieldsSchema(validators.Schema):
         chained_validators = [validators.FieldsMatch('passwd', 'passwd2')]
 
-
     form = widgets.TableForm(fields=Fields(), validator=FieldsSchema())
 
     def test_goodvalues(self):
@@ -486,7 +613,6 @@ class TestSchemaValidationWithChildWidgetsValidators:
                       passwd2="fado")
 
         values, errors = catch_validation_errors(self.form, values)
-        print values
         assert values['age'] == 99
         assert not errors.keys()
 
@@ -494,15 +620,12 @@ class TestSchemaValidationWithChildWidgetsValidators:
         values = dict(name=u'Jos\xc3\xa9', age="ninetynine", passwd="fado",
                       passwd2="fado")
         values, errors = catch_validation_errors(self.form, values)
-        print values, errors
         assert "age" in errors.keys()
 
-
     def test_widget_validator_and_schema_failure(self):
         values = dict(name=u'Jos\xc3\xa9', age="ninetynine", passwd="fado",
                       passwd2="fadO")
         values, errors = catch_validation_errors(self.form, values)
-        print values, errors
         assert "age" in errors.keys()
         assert "passwd2" in errors.keys()
 
@@ -558,7 +681,7 @@ def test_dict_as_validator():
 
 def test_params_doc():
     """Tests params_doc are picked from all bases giving priority to the
-    widget's own"""
+    widget's own."""
     class BaseA(widgets.Widget):
         params_doc = {'a':1}
     class BaseB(widgets.Widget):
@@ -579,3 +702,50 @@ def test_selectfield_with_with_non_iterable_option_elements():
     assert '<option selected="selected" value="python">' in output
     assert '<option value="java">' in output
     assert '<option value="pascal">' in output
+
+def test_calendardatepicker():
+    """CalendarDatePicker widget is wrapped in a SPAN element with proper class
+    """
+    w = widgets.CalendarDatePicker(
+       name = 'test_cdp',
+       field_class = 'my_cdp',
+       label='Test',
+       format='%m/%d/%Y',
+       validator = validators.DateConverter(format="mm/dd/yyyy")
+    )
+    output = w.render(format='xhtml')
+    print output
+    assert '<span class="my_cdp">' in output
+
+def test_calendardatepicker():
+    """CalendarDatePicker widget is wrapped in SPAN element with proper class
+    """
+    w = widgets.CalendarDatePicker(
+       name = 'test_cdp',
+       field_class = 'cdp',
+       label='Test',
+       format='%m/%d/%Y',
+       validator = validators.DateConverter(format="mm/dd/yyyy")
+    )
+    output = w.render(format='xhtml')
+    assert '<span class="cdp">' in output
+
+def test_autocompletefield():
+    """AutoCompleteField widget is wrapped in SPAN element with proper class/id
+    """
+    w = widgets.AutoCompleteField(name="my_acf",
+        field_class='acf',
+        search_controller="/search",
+        search_param="test", result_name="test")
+    output = w.render(format='xhtml')
+    assert '<span id="my_acf" class="acf">' in output
+
+def test_autocompletetextfield():
+    """AutoCompleteTextField widget is wrapped in SPAN element with proper class/id
+    """
+    w = widgets.AutoCompleteTextField(name="my_actf",
+        field_class='actf',
+        search_controller="/search",
+        search_param="test", result_name="test")
+    output = w.render(format='xhtml')
+    assert '<span class="actf">' in output
